
import warnings

from django.conf.urls import include, patterns, url
from django.utils import six
from django.utils.functional import cached_property
from leonardo.utils.settings import (get_conf_from_module,
                                     get_leonardo_modules, get_loaded_modules,
                                     merge)
from leonardo.utils import is_leonardo_module
from importlib import import_module  # noqa
from django.utils.module_loading import module_has_submodule  # noqa
from leonardo.decorators import require_auth, _decorate_urlconf
from leonardo.conf import Default

# use leonardo instead
default = Default()


class Leonardo(object):

    default = default

    MODULES_AUTOLOAD = True

    def load_modules(self):
        """find all leonardo modules from environment"""
        if self.MODULES_AUTOLOAD:
            self.add_modules(get_leonardo_modules())
        return self.modules

    @property
    def is_loaded(self):
        return True if hasattr(self, '_modules') else False

    @property
    def modules(self):
        """loaded modules
        auto populated if is not present
        """
        return self._modules

    def set_modules(self, modules):
        """setter for modules"""
        self._modules = modules

    def add_modules(self, modules):
        """Merge new modules to loaded modules"""
        merged_modules = merge(modules, self.modules)
        self.set_modules(merged_modules)

    def get_modules(self, modules=None):
        """load configuration for all modules"""
        if not hasattr(self, "loaded_modules"):
            self.loaded_modules = get_loaded_modules(modules or self.modules)
        return self.loaded_modules

    def get_modules_as_list(self):
        return [module_cfg for mod, module_cfg in self.get_modules()]

    def get_app_modules(self, apps):
        """return array of imported leonardo modules for apps
        """
        modules = getattr(self, "_modules", [])

        if not modules:
            from django.utils.module_loading import module_has_submodule

            # Try importing a modules from the module package
            package_string = '.'.join(['leonardo', 'module'])

            for app in apps:
                try:
                    # check if is not full app
                    _app = import_module(app)
                except ImportError:
                    _app = False

                if module_has_submodule(import_module(package_string), app) or _app:
                    if _app:
                        mod = _app
                    else:
                        mod = import_module('.{0}'.format(app), package_string)
                    if mod:
                        modules.append(mod)
                        continue

                warnings.warn('%s was skipped because app was '
                              'not found in PYTHONPATH' % app)

            self._modules = modules
        return self._modules

    @cached_property
    def urlpatterns(self):
        '''load and decorate urls from all modules
        then store it as cached property for less loading
        '''
        urlpatterns = []
        # load all urls
        # support .urls file and urls_conf = 'elephantblog.urls' on default module
        # decorate all url patterns if is not explicitly excluded
        for mod in leonardo.modules:
            # TODO this not work
            if is_leonardo_module(mod):

                conf = get_conf_from_module(mod)

                if module_has_submodule(mod, 'urls'):
                    urls_mod = import_module('.urls', mod.__name__)
                    if hasattr(urls_mod, 'urlpatterns'):
                        # if not public decorate all

                        if conf['public']:
                            urlpatterns += urls_mod.urlpatterns
                        else:
                            _decorate_urlconf(urls_mod.urlpatterns,
                                              require_auth)
                            urlpatterns += urls_mod.urlpatterns
        # avoid circural dependency
        # TODO use our loaded modules instead this property
        from django.conf import settings
        for urls_conf, conf in six.iteritems(getattr(settings, 'MODULE_URLS', {})):
            # is public ?
            try:
                if conf['is_public']:
                    urlpatterns += \
                        patterns('',
                                 url(r'', include(urls_conf)),
                                 )
                else:
                    _decorate_urlconf(
                        url(r'', include(urls_conf)),
                        require_auth)
                    urlpatterns += patterns('',
                                            url(r'', include(urls_conf)))
            except Exception as e:
                raise Exception('raised %s during loading %s' % (str(e), urls_conf))
        return urlpatterns

    _instance = None

    def __new__(cls, *args, **kwargs):
        """A singleton implementation of Leonardo. There can be only one.
        """
        if not cls._instance:
            cls._instance = super(Leonardo, cls).__new__(cls, *args, **kwargs)
        return cls._instance

leonardo = Leonardo()




import codecs
import os

from dbtemplates.conf import settings
from dbtemplates.models import Template
from django import VERSION
from django.contrib.sites.models import Site
from leonardo.utils.settings import merge

ALWAYS_ASK, FILES_TO_DATABASE, DATABASE_TO_FILES = ('0', '1', '2')

# support for other template backends

DIRS = []

if VERSION[:2] < (1, 8):
    from django.template.loaders.app_directories import app_template_dirs
    DIRS = settings.TEMPLATE_DIRS
else:
    from django.template.utils import get_app_template_dirs
    from django.template.loader import _engine_list
    for engine in _engine_list():
        DIRS.extend(engine.dirs)
    app_template_dirs = get_app_template_dirs('templates')


def get_or_create_template(template_name, extension='.html', app_first=False,
                           force=True, delete=False, prefix='', notfix=None):

    site = Site.objects.get_current()

    if not extension.startswith("."):
        extension = ".%s" % extension

    if app_first:
        tpl_dirs = merge(app_template_dirs, DIRS)
    else:
        tpl_dirs = merge(DIRS, app_template_dirs)
    templatedirs = [d for d in tpl_dirs if os.path.isdir(d)]

    for templatedir in templatedirs:
        for dirpath, subdirs, filenames in os.walk(templatedir):
            for f in [f for f in filenames
                      if f.endswith(extension) and not f.startswith(".")]:
                path = os.path.join(dirpath, f)
                name = path.split(templatedir)[1]
                if name.startswith('/'):
                    name = name[1:]
                if template_name in name and prefix in path:
                    if notfix and notfix in path:
                        continue
                    try:
                        t = Template.on_site.get(name__exact=name)
                    except Template.DoesNotExist:
                        t = Template(name=name,
                                     content=codecs.open(path, "r").read())
                        t.save(sync_themes=False)
                        t.sites.add(site)

                    else:
                        if force:
                            t.content = codecs.open(path, 'r').read()
                            t.save(sync_themes=False)
                            t.sites.add(site)
                    return t
    return None




from __future__ import absolute_import

from horizon import Horizon
import horizon_contrib
from django.conf.urls import include, patterns, url

# horizon and feinCMS
urlpatterns = patterns('',
                       url(r'', include(Horizon._lazy_urls)),
                       url(r'', include(horizon_contrib.urls)),
                       )



# -#- coding: utf-8 -#-

from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import (Resolver404, resolve)
from django.db import models
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.functional import curry as partial
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from feincms.utils import get_object
from leonardo.module.web.models import Widget
from leonardo.module.web.widgets.forms import WidgetUpdateForm

try:
    from feincms.content.application.models import ApplicationContent
except:
    from feincms.apps import ApplicationContent


class ApplicationWidget(Widget, ApplicationContent):

    icon = "fa fa-plug"

    @classmethod
    def initialize_type(cls, APPLICATIONS):
        for i in APPLICATIONS:
            if not 2 <= len(i) <= 3:
                raise ValueError(
                    "APPLICATIONS must be provided with tuples containing at"
                    " least two parameters (urls, name) and an optional extra"
                    " config dict")

            urls, name = i[0:2]

            if len(i) == 3:
                app_conf = i[2]

                if not isinstance(app_conf, dict):
                    raise ValueError(
                        "The third parameter of an APPLICATIONS entry must be"
                        " a dict or the name of one!")
            else:
                app_conf = {}

            cls.ALL_APPS_CONFIG[urls] = {
                "urls": urls,
                "name": name,
                "config": app_conf
            }

        cls.add_to_class(
            'urlconf_path',
            models.CharField(_('application'), max_length=100, choices=[
                (c['urls'], c['name']) for c in cls.ALL_APPS_CONFIG.values()])
        )

        class ApplicationContentItemEditorForm(WidgetUpdateForm):
            app_config = {}
            custom_fields = {}

            def __init__(self, *args, **kwargs):
                super(ApplicationContentItemEditorForm, self).__init__(
                    *args, **kwargs)

                instance = kwargs.get("instance", None)

                if instance:
                    try:
                        # TODO use urlconf_path from POST if set
                        # urlconf_path = request.POST.get('...urlconf_path',
                        #     instance.urlconf_path)
                        self.app_config = cls.ALL_APPS_CONFIG[
                            instance.urlconf_path]['config']
                    except KeyError:
                        self.app_config = {}

                    self.custom_fields = {}
                    admin_fields = self.app_config.get('admin_fields', {})

                    if isinstance(admin_fields, dict):
                        self.custom_fields.update(admin_fields)
                    else:
                        get_fields = get_object(admin_fields)
                        self.custom_fields.update(
                            get_fields(self, *args, **kwargs))

                    params = self.instance.parameters
                    for k, v in self.custom_fields.items():
                        v.initial = params.get(k)
                        self.fields[k] = v
                        if k in params:
                            self.fields[k].initial = params[k]

    def render_content(self, options):
        data = {
            'widget': self,
            'request': options.get('request'),
        }
        context = RequestContext(options.get('request'), data)

        context['content'] = getattr(
            self, 'rendered_result', '')

        return render_to_string(self.get_template, context)

    def process(self, request, **kw):
        page_url = self.parent.get_absolute_url()

        if "path_mapper" in self.app_config:
            path_mapper = get_object(self.app_config["path_mapper"])
            path, page_url = path_mapper(
                request.path,
                page_url,
                appcontent_parameters=self.parameters
            )
        else:
            path = request._feincms_extra_context['extra_path']

        # Resolve the module holding the application urls.
        urlconf_path = self.app_config.get('urls', self.urlconf_path)

        try:
            fn, args, kwargs = resolve(path, urlconf_path)
        except (ValueError, Resolver404):
            raise Resolver404(str('Not found (resolving %r in %r failed)') % (
                path, urlconf_path))

        # Variables from the ApplicationContent parameters are added to request
        # so we can expose them to our templates via the appcontent_parameters
        # context_processor
        request._feincms_extra_context.update(self.parameters)
        request._feincms_extra_context.update({'widget': self})

        # Save the application configuration for reuse elsewhere
        request._feincms_extra_context.update({
            'app_config': dict(
                self.app_config,
                urlconf_path=self.urlconf_path,
            ),
        })

        view_wrapper = self.app_config.get("view_wrapper", None)
        if view_wrapper:
            fn = partial(
                get_object(view_wrapper),
                view=fn,
                appcontent_parameters=self.parameters
            )

        output = fn(request, *args, **kwargs)

        # handle django rest framework as external application
        if hasattr(output, 'renderer_context'):
            output.context_data = output.renderer_context
            output.standalone = True

        if isinstance(output, HttpResponse):

            # update context
            if hasattr(output, 'context_data'):
                output.context_data['widget'] = self
            else:
                output.context_data = {'widget': self}

            if self.send_directly(request, output):
                return output
            elif output.status_code == 200:

                if self.unpack(request, output) and 'view' in kw:
                    # Handling of @unpack and UnpackTemplateResponse
                    kw['view'].template_name = output.template_name
                    kw['view'].request._feincms_extra_context.update(
                        output.context_data)

                else:
                    # If the response supports deferred rendering, render the
                    # response right now. We do not handle template response
                    # middleware.
                    if hasattr(output, 'render') and callable(output.render):
                        output.render()

                    self.rendered_result = mark_safe(
                        output.content.decode('utf-8'))

                self.rendered_headers = {}

                # Copy relevant headers for later perusal
                for h in ('Cache-Control', 'Last-Modified', 'Expires'):
                    if h in output:
                        self.rendered_headers.setdefault(
                            h, []).append(output[h])

        elif isinstance(output, tuple) and 'view' in kw:
            kw['view'].template_name = output[0]
            kw['view'].request._feincms_extra_context.update(output[1])
            # our hack
            # no template and view change and save content for our widget
            context = output[1]
            context['widget'] = self
            self.rendered_result = render_to_string(
                output[0], RequestContext(request, context))
        else:
            self.rendered_result = mark_safe(output)

        # here is the magic !
        # return renderered parent template !
        context = RequestContext(request, {})
        return render_to_response(
            self.parent.theme.template,
            context
            )

    class Meta:
        abstract = True
        verbose_name = _("External application")
        verbose_name_plural = _('External applications')



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from leonardo.module.web.models import Widget


LOGIN_TYPE_CHOICES = (
    (1, _("Admin")),
    (2, _("Public")),
)


class UserLoginWidget(Widget):
    type = models.PositiveIntegerField(verbose_name=_(
        "type"), choices=LOGIN_TYPE_CHOICES, default=2)

    class Meta:
        abstract = True
        verbose_name = _("user login")
        verbose_name_plural = _("user logins")




from django.conf import settings
from django.utils import six


def get_all_widget_classes():
    """returns collected Leonardo Widgets

    if not declared in settings is used __subclasses__
    which not supports widget subclassing

    """
    from leonardo.module.web.models import Widget
    _widgets = getattr(settings,
                       'WIDGETS', Widget.__subclasses__())
    widgets = []
    if isinstance(_widgets, dict):
        for group, widget_cls in six.iteritems(_widgets):
            widgets.extend(widget_cls)
    elif isinstance(_widgets, list):
        widgets = _widgets
    return widgets


def get_grouped_widgets(feincms_object, request=None):
    '''returns tuple(choices, grouped, ungrouped)

    requires feincms_object for getting content types

    request optionaly for checking permissions, but not required

    grouped = {'web': (id, label, icon)}
    '''

    grouped = {}
    ungrouped = []
    choices = []

    for ct in feincms_object._feincms_content_types:
        # Skip cts that we shouldn't be adding anyway
        opts = ct._meta
        # check permissions
        if request and request.user:
            from django.contrib.auth import get_permission_codename
            perm = opts.app_label + "." + \
                get_permission_codename('add', opts)
            if not request.user.has_perm(perm):
                continue

        ct_info = ('.'.join([ct._meta.app_label,
                             ct.__name__.lower()]),
                   ct._meta.verbose_name,
                   ct.get_widget_icon)
        if hasattr(ct, 'optgroup'):
            if ct.optgroup in grouped:
                grouped[ct.optgroup].append(ct_info)
            else:
                grouped[ct.optgroup] = [ct_info]
        else:
            ungrouped.append(ct_info)
        choices.append(ct_info)

    return choices, grouped, ungrouped


def find_widget_class(name):

    for w_cls in get_all_widget_classes():
        if name.lower() in w_cls.__name__.lower():
            return w_cls
    return None




import os

from django.db import models
from django.utils.translation import ugettext_lazy as _
from feincms.module.page.extensions.navigation import (NavigationExtension,
                                                       PagePretender)
from feincms.translations import Translation

from .imagemodels import Image
from .filemodels import File
from .foldermodels import Folder

MEDIA_FILE_EXTENSIONS = {
    'media.document': ['.pdf', '.xls'],
    'media.vector': ['.svg', '.eps', ],
    'media.video': ['.dv', '.mov', '.mp4', '.avi', '.wmv', ],
    'media.flash': ['.swf']
}


class MediaMixin(object):

    @classmethod
    def matches_file_type(cls, iname, ifile=None, request=None):
        # the extensions we'll recognise for this file type
        # (majklk): TODO move to settings or live config
        filename_extensions = getattr(cls, 'filename_extensions', '*')
        ext = os.path.splitext(iname)[1].lower()
        return ext in filename_extensions


class MediaTranslationMixin(object):

    original_filename = models.CharField(
        _('original filename'), max_length=255, blank=True, null=True)
    name = models.CharField(max_length=255, default="", blank=True,
                            verbose_name=_('name'))
    description = models.TextField(null=True, blank=True,
                                   verbose_name=_('description'))


class FolderTranslation(Translation(Folder)):

    """
    Translated Document
    """

    name = models.CharField(max_length=255, default="", blank=True,
                            verbose_name=_('name'))

    class Meta:
        verbose_name = ("Folder translation")
        verbose_name_plural = ('Folder translations')
        app_label = 'media'


class Document(MediaMixin, File):

    filename_extensions = MEDIA_FILE_EXTENSIONS['media.document']

    class Meta:
        verbose_name = ("document")
        verbose_name_plural = ('documents')


class DocumentTranslation(Translation(Document), MediaTranslationMixin):

    """
    Translated Document
    """

    class Meta:
        verbose_name = ("document translation")
        verbose_name_plural = ('document translations')
        app_label = 'media'


class Vector(MediaMixin, File):

    filename_extensions = MEDIA_FILE_EXTENSIONS['media.vector']

    class Meta:
        verbose_name = ("vector")
        verbose_name_plural = ('vetors')


class VectorTranslation(Translation(Vector), MediaTranslationMixin):

    """
    Translated Vector
    """

    class Meta:
        verbose_name = ("vector translation")
        verbose_name_plural = ('vector translations')
        app_label = 'media'


class Video(MediaMixin, File):

    filename_extensions = MEDIA_FILE_EXTENSIONS['media.video']

    class Meta:
        verbose_name = ("video")
        verbose_name_plural = ('videos')


class VideoTranslation(Translation(Video), MediaTranslationMixin):

    """
    Translated Video
    """

    class Meta:
        verbose_name = ("video translation")
        verbose_name_plural = ('video translations')
        app_label = 'media'


class Flash(MediaMixin, File):

    filename_extensions = MEDIA_FILE_EXTENSIONS['media.flash']

    class Meta:
        verbose_name = ("flash video")
        verbose_name_plural = ('flash videos')


class ImageTranslation(Translation(Image)):

    name = models.CharField(max_length=255, default="", blank=True,
                            verbose_name=_('name'))
    default_alt_text = models.CharField(max_length=255, default="", blank=True,
                                        verbose_name=_('default alt text'))

    default_caption = models.CharField(max_length=255, default="", blank=True,
                                       verbose_name=_('default caption'))

    description = models.TextField(null=True, blank=True,
                                   verbose_name=_('description'))

    class Meta:
        verbose_name = ("image translation")
        verbose_name_plural = ('image translations')
        app_label = 'media'


class MediaCategoriesNavigationExtension(NavigationExtension):
    name = _('All media categories')

    def children(self, page, **kwargs):
        base_url = page.get_absolute_url()
        category_list = Folder.objects.filter(parent=None)
        for category in category_list:
            subchildren = []
            for subcategory in category.media_folder_children.all():
                subchildren.append(PagePretender(
                    title=subcategory,
                    url='%s%s/%s/' % (base_url, category.name, subcategory.name),
                    level=5
                ))
            yield PagePretender(
                title=category,
                url='%s%s/' % (base_url, category.name),
                children=subchildren,
                level=5
            )




from __future__ import absolute_import

from django.apps import AppConfig

from django.utils.functional import lazy
from django.utils import six


class LeonardoConfig(AppConfig):
    name = 'leonardo'
    verbose_name = "Leonardo"

    def ready(self):

        from django.conf import settings
        from django.template import defaulttags

        # monkey path url tag
        from leonardo.utils.urlresolvers import render
        defaulttags.URLNode.render = render

        # monkey path django reverse
        from django.core import urlresolvers
        from leonardo.module.web.widget.application.reverse import reverse

        urlresolvers.reverse = reverse
        urlresolvers.reverse_lazy = lazy(reverse, six.text_type)

        # path settings for support functionality without hell
        # this needs restart for propagation some keys
        from horizon.conf import HORIZON_CONFIG
        from horizon import conf as horizon_conf
        try:
            # optionaly copy all live configuration to horizon/leonardo
            from constance import config

            for k in dir(config):
                if k not in HORIZON_CONFIG:
                    HORIZON_CONFIG[k] = getattr(config, k)
        except:
            pass
        else:
            horizon_conf.HORIZON_CONFIG = HORIZON_CONFIG

        # use our debug 404 for quick build scaffold of site
        try:
            from django.views import debug
            from leonardo.views.debug import technical_404_response
            debug.technical_404_response = technical_404_response
        except:
            pass

        # now we patch all models with absolute url overrides
        # this is important because Django path get_absolute_url when model
        # is imported ! this behaviour breaks our strategy
        from django.apps import apps
        from importlib import import_module  # noqa

        for model, method in six.iteritems(settings.ABSOLUTE_URL_OVERRIDES):
            try:
                mod_name = model.split('.')[0]
                model_cls = apps.get_model(mod_name, model.split('.')[-1])
                if callable(method):
                    _method = method
                else:
                    _mod = import_module(".".join(method.split('.')[:-1]))
                    _method = getattr(_mod, method.split('.')[-1])

                setattr(model_cls, 'get_absolute_url', _method)

            except Exception as e:
                raise e

        # patch compress
        from leonardo.utils.compress_patch import compress_monkey_patch
        compress_monkey_patch()



# -#- coding: utf-8 -#-

from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import get_language_from_request
from leonardo.module.nav.forms import NavigationForm
from leonardo.module.web.models import Page, Widget


class SimpleLinkWidget(Widget):

    feincms_item_editor_form = NavigationForm
    page = models.ForeignKey(Page, verbose_name=_(
        "Linked page"), related_name="simplelink_node")

    class Meta:
        abstract = True
        verbose_name = _("Simple link")
        verbose_name_plural = _('Simple links')

    def render_content(self, options):
        request = options['request']

        return render_to_string(self.get_template_name(), {
            'widget': self,
            'request': request,
        })




from __future__ import absolute_import

from django.conf.urls import include, patterns, url
from horizon_contrib.forms.views import (CreateView,
                                         UpdateView)
from horizon_contrib.generic.views import GenericIndexView

# override native horizon-contrib views
GenericIndexView.template_name = "leonardo/common/_index.html"
CreateView.template_name = "leonardo/common/modal.html"
UpdateView.template_name = "leonardo/common/modal.html"


urlpatterns = patterns('',
                       url(r'^page/', include('leonardo.module.web.page.urls')),
                       url(r'^widget/', include('leonardo.module.web.widgets.urls')),
                       )



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from leonardo.module.web.models import Widget

VERSION_CHOICES = (
    ('10.0.0', _('version 10 and newer')),
    ('9.0.0', _('version 9 and newer')),
)

VARS_DEFAULT = """var vars = {
}
"""

PARAMS_DEFAULT = """var params = {
}
"""

ATTRS_DEFAULT = """var attrs = {
}
"""


class FlashObjectWidget(Widget):
    file = models.ForeignKey("media.Flash", verbose_name=_(
        "flash file"), related_name="%(app_label)s_%(class)s_categories")
    version = models.CharField(
        max_length=255, verbose_name=_("flash version"), choices=VERSION_CHOICES)
    width = models.CharField(
        max_length=255, verbose_name=_("width"), default='640')
    height = models.CharField(
        max_length=255, verbose_name=_("height"), default='480')
    vars = models.TextField(verbose_name=_("variables"), default=VARS_DEFAULT)
    params = models.TextField(
        verbose_name=_("parameters"), default=PARAMS_DEFAULT)
    attrs = models.TextField(
        verbose_name=_("attributes"), default=ATTRS_DEFAULT)

    class Meta:
        abstract = True
        verbose_name = _("flash object")
        verbose_name_plural = _('flash objects')




from django.core.exceptions import PermissionDenied


def check_files_edit_permissions(request, files):
    for f in files:
        if not f.has_edit_permission(request):
            raise PermissionDenied


def check_folder_edit_permissions(request, folders):
    for f in folders:
        if not f.has_edit_permission(request):
            raise PermissionDenied
        check_files_edit_permissions(request, f.files)
        check_folder_edit_permissions(request, f.media_folder_children.all())


def check_files_read_permissions(request, files):
    for f in files:
        if not f.has_read_permission(request):
            raise PermissionDenied


def check_folder_read_permissions(request, folders):
    for f in folders:
        if not f.has_read_permission(request):
            raise PermissionDenied
        check_files_read_permissions(request, f.files)
        check_folder_read_permissions(request, f.media_folder_children.all())


def userperms_for_request(item, request):
    r = []
    ps = ['read', 'edit', 'add_children']
    for p in ps:
        attr = "has_%s_permission" % p
        if hasattr(item, attr):
            x = getattr(item, attr)(request)
            if x:
                r.append(p)
    return r




INTERNAL_IPS = ['10.10.10.1', '127.0.0.1']
DEBUG_TOOLBAR_PANELS = [
    'debug_toolbar.panels.versions.VersionsPanel',
    'debug_toolbar.panels.timer.TimerPanel',
    'debug_toolbar.panels.settings.SettingsPanel',
    'debug_toolbar.panels.headers.HeadersPanel',
    'debug_toolbar.panels.request.RequestPanel',
    'debug_toolbar.panels.sql.SQLPanel',
    'debug_toolbar.panels.staticfiles.StaticFilesPanel',
    'debug_toolbar.panels.templates.TemplatesPanel',
    'debug_toolbar.panels.cache.CachePanel',
    'debug_toolbar.panels.signals.SignalsPanel',
    'debug_toolbar.panels.logging.LoggingPanel',
    'debug_toolbar.panels.redirects.RedirectsPanel',
]





from crispy_forms.bootstrap import (Accordion, AccordionGroup, FieldWithButtons,
                                    StrictButton, Tab)
from crispy_forms.layout import Field, HTML, Layout
from crispy_forms.helper import FormHelper
from django.utils.translation import ugettext_lazy as _
from haystack.forms import ModelSearchForm


class SearchForm(ModelSearchForm):

    def _wrap_all(self):
        # stylung
        self.helper.filter(
            str, max_level=4).wrap(
            Field, css_class="form-control")

    def __init__(self, *args, **kwargs):
        super(SearchForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.form_tag = False
        self.helper.layout = Layout(
            Accordion(
                Field('q', placeholder=_("Search"), css_class="form-control", wrapper_class='form-group'),
                StrictButton(_("Search"), type='submit', css_class="btn btn-default"),
                AccordionGroup(_('Options'),
                               Field('models')
                               )
            )
        )




from __future__ import absolute_import

from .widget import *

from .const import *
from .models import Page


FEINCMS_REVERSE_MONKEY_PATCH = False

LEONARDO_FRONTEND_EDITING = True

Page.register_templates(*PAGE_TEMPLATES)

THUMBNAIL_DEBUG = True



from django.conf.urls import patterns, url


urlpatterns = patterns('leonardo.module.media.server.views',
                       url(r'^(?P<path>.*)$', 'serve_protected_thumbnail',),
                       )




from ..models import Clipboard


def discard_clipboard(clipboard):
    clipboard.files.clear()


def delete_clipboard(clipboard):
    for file_obj in clipboard.files.all():
        file_obj.delete()


def get_user_clipboard(user):
    if user.is_authenticated():
        clipboard = Clipboard.objects.get_or_create(user=user)[0]
        return clipboard


def move_file_to_clipboard(files, clipboard):
    count = 0
    for file_obj in files:
        if clipboard.append_file(file_obj):
            file_obj.folder = None
            file_obj.save()
            count += 1
    return count


def move_files_from_clipboard_to_folder(clipboard, folder):
    return move_files_to_folder(clipboard.files.all(), folder)


def move_files_to_folder(files, folder):
    for file_obj in files:
        file_obj.folder = folder
        file_obj.save()
    return True




from __future__ import unicode_literals

from django.conf import settings
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.forms.models import fields_for_model
from django.template import loader, RequestContext
from django.template.loader import render_to_string
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from feincms.admin.item_editor import FeinCMSInline
from feincms.models import Base as FeinCMSBase
from leonardo.utils.templates import find_all_templates, template_choices

from ..const import *
from ..widgets.forms import WIDGETS, WidgetUpdateForm
from ..widgets.mixins import ListWidgetMixin, ContentProxyWidgetMixin
from django.utils.functional import cached_property
from leonardo.utils.memoized import widget_memoized


class WidgetInline(FeinCMSInline):
    form = WidgetUpdateForm

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "base_theme":
            queryset = WidgetBaseTheme.objects.all()
            kwargs["queryset"] = queryset.exclude(name__startswith="_")
            kwargs["initial"] = queryset.first()
        if db_field.name == "content_theme":
            queryset = WidgetContentTheme.objects.filter(
                widget_class=self.model.__name__)
            kwargs["queryset"] = queryset.exclude(name__startswith="_")
            kwargs["initial"] = queryset.first()
        form_field = super(WidgetInline, self).formfield_for_foreignkey(
            db_field, request, **kwargs)
        # bootstrap field
        form_field.widget.attrs['class'] = 'form-control'
        return form_field

    def __init__(self, *args, **kwargs):
        super(WidgetInline, self).__init__(*args, **kwargs)

        self.fieldsets = [
            (None, {
                'fields': [
                    list(self.model.fields())
                ],
            }),
            (_('Theme'), {
                'fields': [
                    ('label', 'base_theme', 'content_theme',
                     'layout', 'align', 'enabled', 'color_scheme'),
                ],
            }),
            (_('Effects'), {
                'fields': [
                    ('enter_effect_style', 'enter_effect_duration',
                     'enter_effect_delay', 'enter_effect_offset',
                     'enter_effect_iteration', 'enabled',),
                ],
            }),
        ]


@python_2_unicode_compatible
class WidgetDimension(models.Model):

    widget_type = models.ForeignKey(ContentType)
    widget_id = models.PositiveIntegerField()
    widget_object = generic.GenericForeignKey('widget_type', 'widget_id')

    size = models.CharField(
        verbose_name="Size", max_length=20, choices=DISPLAY_SIZE_CHOICES, default='md')
    width = models.IntegerField(verbose_name=_("Width"),
                                choices=COLUMN_CHOICES, default=DEFAULT_WIDTH)
    height = models.IntegerField(verbose_name=_("Height"),
                                 choices=ROW_CHOICES, default=0)
    offset = models.IntegerField(verbose_name=_("Offset"),
                                 choices=COLUMN_CHOICES, default=0)

    @cached_property
    def classes(self):
        classes = []
        classes.append('col-{}-{}'.format(self.size, self.width))
        if self.height != 0:
            classes.append('row-{}-{}'.format(self.size, self.height))
        classes.append('col-{}-offset-{}'.format(self.size, self.offset))
        return ' '.join(classes)

    def __str__(self):
        return "{0} - {1}".format(self.widget_type, self.classes)

    class Meta:
        verbose_name = _("Widget dimension")
        verbose_name_plural = _("Widget dimensions")


@python_2_unicode_compatible
class WidgetContentTheme(models.Model):

    name = models.CharField(
        verbose_name=_("Name"), max_length=255, null=True, blank=True)
    label = models.CharField(
        verbose_name=_("Title"), max_length=255, null=True, blank=True)
    template = models.ForeignKey(
        'dbtemplates.Template', verbose_name=_('Content template'),
        related_name='content_templates', limit_choices_to={'name__startswith': "widget/"})
    style = models.TextField(verbose_name=_('Content style'), blank=True)
    widget_class = models.CharField(
        verbose_name=_('Widget class'), max_length=255)

    def __str__(self):
        return self.label or str(self._meta.verbose_name + ' %s' % self.pk)

    class Meta:
        verbose_name = _("Widget content theme")
        verbose_name_plural = _("Widget content themes")


@python_2_unicode_compatible
class WidgetBaseTheme(models.Model):

    name = models.CharField(
        verbose_name=_("Name"), max_length=255, null=True, blank=True)
    label = models.CharField(
        verbose_name=_("Title"), max_length=255, null=True, blank=True)
    template = models.ForeignKey(
        'dbtemplates.Template', verbose_name=_('Base template'),
        related_name='base_templates', limit_choices_to={'name__startswith': "base/widget/"})

    style = models.TextField(verbose_name=_('Base style'), blank=True)

    def __str__(self):
        return self.label or str(self._meta.verbose_name + ' %s' % self.pk)

    class Meta:
        verbose_name = _("Widget base theme")
        verbose_name_plural = _("Widget base themes")


from ..widgets.const import ENTER_EFFECT_CHOICES, WIDGET_COLOR_SCHEME_CHOICES


@python_2_unicode_compatible
class Widget(FeinCMSBase):

    feincms_item_editor_inline = WidgetInline

    prerendered_content = models.TextField(
        verbose_name=_('prerendered content'), blank=True)
    enabled = models.NullBooleanField(
        verbose_name=_('Is visible?'), default=True)
    label = models.CharField(
        verbose_name=_("Title"), max_length=255, null=True, blank=True)
    base_theme = models.ForeignKey(
        WidgetBaseTheme, verbose_name=_('Base theme'),
        related_name="%(app_label)s_%(class)s_related")
    content_theme = models.ForeignKey(
        WidgetContentTheme, verbose_name=_('Content theme'),
        related_name="%(app_label)s_%(class)s_related")
    layout = models.CharField(
        verbose_name=_("Layout"), max_length=25,
        default='inline', choices=WIDGET_LAYOUT_CHOICES)
    align = models.CharField(
        verbose_name=_("Alignment"), max_length=25,
        default='left', choices=WIDGET_ALIGN_CHOICES)
    vertical_align = models.CharField(
        verbose_name=_("Vertical Alignment"), max_length=25,
        default='top', choices=VERTICAL_ALIGN_CHOICES)

    # common attributes
    enter_effect_style = models.CharField(
        verbose_name=_("Enter effect style"),
        max_length=25, default='disabled', choices=ENTER_EFFECT_CHOICES)

    enter_effect_duration = models.PositiveIntegerField(verbose_name=_(
        'Enter Effect Duration'), null=True, blank=True)
    enter_effect_delay = models.PositiveIntegerField(null=True, blank=True)
    enter_effect_offset = models.PositiveIntegerField(null=True, blank=True)
    enter_effect_iteration = models.PositiveIntegerField(null=True, blank=True)

    color_scheme = models.CharField(
        verbose_name=_("Color scheme"),
        max_length=25, default='default', choices=WIDGET_COLOR_SCHEME_CHOICES)

    def save(self, created=True, *args, **kwargs):

        if self.pk is None and created:
            self.created = True

        super(Widget, self).save(*args, **kwargs)

        if not self.dimensions.exists() and created:
            WidgetDimension(**{
                'widget_id': self.pk,
                'widget_type': self.content_type,
            }).save()

    def delete(self, *args, **kwargs):
        super(Widget, self).delete(*args, **kwargs)
        [d.delete() for d in self.dimensions]

    class Meta:
        abstract = True
        verbose_name = _("Abstract widget")
        verbose_name_plural = _("Abstract widgets")

    def __str__(self):
        return self.label or (
            '%s<pk=%s, parent=%s<pk=%s, %s>, region=%s,'
            ' ordering=%d>') % (
            self.__class__.__name__,
            self.pk,
            self.parent.__class__.__name__,
            self.parent.pk,
            self.parent,
            self.region,
            self.ordering)

    @cached_property
    def get_ct_name(self):
        """returns content type name with app label
        """
        return ".".join([self._meta.app_label, self._meta.model_name])

    @cached_property
    def content_type(self):
        return ContentType.objects.get_for_model(self)

    def thumb_geom(self):
        return config_value('MEDIA', 'THUMB_MEDIUM_GEOM')

    def thumb_options(self):
        return config_value('MEDIA', 'THUMB_MEDIUM_OPTIONS')

    def get_template_name(self):
        return self.content_theme.template

    @cached_property
    def get_template(self):
        return self.content_theme.template

    def _template_xml_name(self):
        template = 'default'
        return u'widget/%s/%s.xml' % (self.widget_name, template)
    template_xml_name = property(_template_xml_name)

    @property
    def widget_name(self):
        return self.__class__.__name__.lower().replace('widget', '')

    @cached_property
    def get_base_template(self):
        return self.base_theme.template

    @cached_property
    def widget_label(self):
        return self._meta.verbose_name

    @cached_property
    def template_source(self):
        template = loader.get_template(self.content_theme.template)
        return template

    def render(self, **kwargs):
        return self.render_content(kwargs)

    @widget_memoized
    def render_with_cache(self, options):
        """proxy for render_content with memoized

        this method provide best performence for complicated
        widget content like a context navigation
        """
        return self.render_content(options)

    def render_content(self, options):

        context = RequestContext(options['request'], {
            'widget': self,
            'base_template': self.get_base_template,
            'request': options['request'],
        })

        # handle widget render error
        try:
            rendered_content = self.template_source.render(context)
        except Exception as e:
            if settings.DEBUG:
                raise e
            rendered_content = self.render_error(context, e)
        return rendered_content

    def render_error(self, context, exception):
        return render_to_string("widget/error.html", {
            'widget': self,
            'request': context['request'],
            'context': context,
            'error': str(exception),
        })

    def render_response(self, context={}):
        '''just render to string shortcut for less imports'''
        return render_to_string(self.get_template_name, context)

    @cached_property
    def model_cls(self):
        return self.__class__.__name__

    @cached_property
    def dimensions(self):
        return WidgetDimension.objects.filter(
            widget_id=self.pk,
            widget_type=ContentType.objects.get_for_model(self))

    @cached_property
    def get_dimension_classes(self):
        """agreggate all css classes
        """
        classes = []
        for d in self.dimensions:
            classes.append(d.classes)
        return classes

    @cached_property
    def render_content_classes(self):
        """agreggate all css classes
        """
        classes = [
            "text-%s" % self.align,
            'leonardo-content',
            'template-%s' % self.content_theme.name,
            '%s-content-%s' % (self.widget_name, self.content_theme.name)
        ]
        return " ".join(classes)

    @cached_property
    def render_base_classes(self):
        """agreggate all css classes
        """
        classes = self.get_dimension_classes
        classes.append('%s-base-%s' % (self.widget_name, self.base_theme.name))
        classes.append('leonardo-widget')
        classes.append('leonardo-%s-widget' % self.widget_name)
        return " ".join(classes)

    @classmethod
    def templates(cls, choices=False, suffix=True):
        """returns widget templates located in ``templates/widget/widgetname``
        """
        widget_name = cls.__name__.lower().replace('widget', '')

        pattern = 'widget/{0}/'.format(widget_name)
        res = find_all_templates('{0}*'.format(pattern))

        if choices:
            return template_choices(res, suffix=suffix)
        return res

    @classmethod
    def fields(cls):
        widget_fields = [
            f.name for f in Widget._meta.fields]

        return fields_for_model(
            cls, exclude=widget_fields,
            widgets=WIDGETS)

    @property
    def next_ordering(self):
        """return order for creating in content region
        """
        if self.parent:
            return len(getattr(self.parent.content, self.region, [])) + 1
        else:
            return 0

    @cached_property
    def fe_identifier(self):
        """
        Returns an identifier which is understood by the frontend
        editing javascript code. (It is used to find the URL which
        should be used to load the form for every given block of
        content.)
        """
        meta = self.__class__._meta
        return '%s-%s-%s-%s' % (
            meta.app_label,
            meta.model_name,
            self.parent_id,
            self.id,
        )

    @classmethod
    def get_widget_icon(cls):
        return getattr(cls, 'icon', 'fa fa-plus')


class ListWidget(Widget, ListWidgetMixin):

    """Base class for object list widget
    """
    class Meta:
        abstract = True



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import django.forms
from django import template as django_template

register = django_template.Library()


@register.filter
def add_bootstrap_class(field):
    """Add a "form-control" CSS class to the field's widget.

    This is so that Bootstrap styles it properly.
    """
    if not isinstance(field.field.widget, (
        django.forms.widgets.CheckboxInput,
        django.forms.widgets.CheckboxSelectMultiple,
        django.forms.widgets.RadioSelect,
        django.forms.widgets.FileInput,
        str
    )):
        field_classes = set(field.field.widget.attrs.get('class', '').split())
        field_classes.add('form-control')
        field.field.widget.attrs['class'] = ' '.join(field_classes)
    return field


@register.filter
def is_checkbox(field):
    return isinstance(field.field.widget, django.forms.CheckboxInput)


@register.filter
def is_multiple_checkbox(field):
    return isinstance(field.field.widget, django.forms.CheckboxSelectMultiple)


@register.filter
def is_radio(field):
    return isinstance(field.field.widget, django.forms.RadioSelect)


@register.filter
def is_file(field):
    return isinstance(field.field.widget, django.forms.FileInput)


@register.filter
def add_item_url(field):
    if hasattr(field.field.widget, 'get_add_item_url'):
        return field.field.widget.get_add_item_url()
    return None


@register.filter
def wrapper_classes(field):
    classes = []
    if is_multiple_checkbox(field):
        classes.append('multiple-checkbox')
    return ' '.join(classes)




from django import forms
from django_select2.forms import *

from ..models import Page, PageColorScheme, PageTheme
from .widgets import (PageColorSchemeSelectWidget, PageSelectWidget,
                      PageThemeSelectWidget)


class Field(forms.ModelChoiceField):

    def __init__(self, *args, **kwargs):
        super(Field, self).__init__(
            queryset=getattr(self, 'model', Page).objects.all(),
            empty_label='---', *args, **kwargs)

    def label_from_instance(self, obj):
        """
        Coerces ``value`` to a Python data type.
        """
        return obj.tree_label


class PageSelectField(Field):
    '''Page Select2 Field'''
    model = Page
    widget = PageSelectWidget


class PageColorSchemeSelectField(Field):
    '''PageColorScheme Select2 Field'''
    model = PageColorScheme
    widget = PageColorSchemeSelectWidget


class PageThemeSelectField(Field):
    '''PageTheme Select2 Field'''
    model = PageTheme
    widget = PageThemeSelectWidget



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('media', '0002_auto_20150531_2215'),
    ]

    operations = [
        migrations.AlterField(
            model_name='clipboard',
            name='user',
            field=models.ForeignKey(related_name='media_clipboards', verbose_name='user', to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.template import Library

register = Library()


def filer_staticmedia_prefix():
    """
    Returns the string contained in the setting FILER_STATICMEDIA_PREFIX.
    """
    try:
        from .. import settings
    except ImportError:
        return ''
    return settings.FILER_STATICMEDIA_PREFIX
filer_staticmedia_prefix = register.simple_tag(filer_staticmedia_prefix)



#-*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from filer.utils.compatibility import python_2_unicode_compatible

from . import filemodels


@python_2_unicode_compatible
class Clipboard(models.Model):
    user = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'), verbose_name=_(
        'user'), related_name="media_clipboards")
    files = models.ManyToManyField(
        'media.File', verbose_name=_('files'), related_name="in_clipboards",
        through='ClipboardItem')

    def append_file(self, file_obj):
        try:
            # We have to check if file is already in the clipboard as otherwise
            # polymorphic complains
            self.files.get(pk=file_obj.pk)
            return False
        except filemodels.File.DoesNotExist:
            newitem = ClipboardItem(file=file_obj, clipboard=self)
            newitem.save()
            return True

    def __str__(self):
        return "Clipboard %s of %s" % (self.id, self.user)

    class Meta:
        app_label = 'media'
        verbose_name = _('clipboard')
        verbose_name_plural = _('clipboards')


class ClipboardItem(models.Model):
    file = models.ForeignKey('media.File', verbose_name=_('file'))
    clipboard = models.ForeignKey(Clipboard, verbose_name=_('clipboard'))

    class Meta:
        app_label = 'media'
        verbose_name = _('clipboard item')
        verbose_name_plural = _('clipboard items')




ENTER_EFFECT_CHOICES = (
    ('disabled', 'Disabled'),
    ('Attention Seekers', (
        ('bounce', 'Bounce'),
        ('flash', 'Flash'),
        ('pulse', 'Pulse'),
        ("rubberBand", 'RubberBand'),
        ("shake", 'Shake'),
        ("swing", 'Swin)g'),
        ("tada", 'Tada'),
        ("wobble", 'Wobble'),
        ("jello", 'Jello'),
    )
    ),
    ('Bouncing Entrances', (
        ('bounceIn', 'Bounce In'),
        ('bounceInDown', 'Bounce In Down'),
        ('bounceInDown', 'BounceInDown'),
        ('bounceInLeft', 'BounceInLeft'),
        ('bounceInRight', 'BounceInRight'),
        ('bounceInUp', 'BounceInUp'),
    )
    ),
    ('Bouncing Exits', (
        ('bounceIn', 'Bounce In'),
        ('bounceOut', 'BounceOut'),
        ('bounceOutDown', 'BounceOutDown'),
        ('bounceOutLeft', 'BounceOutLeft'),
        ('bounceOutRight', 'BounceOutRight'),
        ('bounceOutUp', 'BounceOutUp'),
    )
    ),
    ('Fading Entrances', (
        ('fadeIn', 'fadeIn'),
        ('fadeInDown', 'fadeInDown'),
        ('fadeInDownBig', 'fadeInDownBig'),
        ('fadeInLeft', 'fadeInLeft'),
        ('fadeInLeftBig', 'fadeInLeftBig'),
        ('fadeInRight', 'fadeInRight'),
        ('fadeInRightBig', 'fadeInRightBig'),
        ('fadeInUp', 'fadeInUp'),
        ('fadeInUpBig', 'fadeInUpBig'),
    )
    ),
    ('Fading Exits', (
        ('fadeOut', 'fadeOut'),
        ('fadeOutDown', 'fadeOutDown'),
        ('fadeOutDownBig', 'fadeOutDownBig'),
        ('fadeOutLeft', 'fadeOutLeft'),
        ('fadeOutLeftBig', 'fadeOutLeftBig'),
        ('fadeOutRight', 'fadeOutRight'),
        ('fadeOutRightBig', 'fadeOutRightBig'),
        ('fadeOutUp', 'fadeOutUp'),
        ('fadeOutUpBig', 'fadeOutUpBig'),
    )
    ),
    ('Flippers', (
        ('flip', 'flip'),
        ('flipInX', 'flipInX'),
        ('flipInY', 'flipInY'),
        ('flipOutX', 'flipOutX'),
        ('flipOutY', 'flipOutY'),
    )
    ),
    ('Lightspeed', (
        ('lightSpeedIn', 'lightSpeedIn'),
        ('lightSpeedOut', 'lightSpeedOut'),
    )
    ),
    ('Rotating Entrances', (
        ('rotateIn', 'rotateIn'),
        ('rotateInDownLeft', 'rotateInDownLeft'),
        ('rotateInDownRight', 'rotateInDownRight'),
        ('rotateInUpLeft', 'rotateInUpLeft'),
        ('rotateInUpRight', 'rotateInUpRight'),
    )
    ),
    ('Rotating Exits', (
        ('rotateOut', 'rotateOut'),
        ('rotateOutDownLeft', 'rotateOutDownLeft'),
        ('rotateOutDownRight', 'rotateOutDownRight'),
        ('rotateOutUpLeft', 'rotateOutUpLeft'),
        ('rotateOutUpRight', 'rotateOutUpRight'),
    )
    ),
    ('Sliding Entrances', (
        ('slideInUp', 'slideInUp'),
        ('slideInDown', 'slideInDown'),
        ('slideInLeft', 'slideInLeft'),
        ('slideInRight', 'slideInRight'),
    )
    ),
    ('Sliding Exits', (
        ('slideOutUp', 'slideOutUp'),
        ('slideOutDown', 'slideOutDown'),
        ('slideOutLeft', 'slideOutLeft'),
        ('slideOutRight', 'slideOutRight'),
    )
    ),
    ('Zoom Entrances', (
        ('slideOutUp', 'slideOutUp'),
        ('slideOutDown', 'slideOutDown'),
        ('slideOutLeft', 'slideOutLeft'),
        ('slideOutRight', 'slideOutRight'),
    )
    ),
    ('Zoom Exits', (
        ('zoomIn', 'zoomIn'),
        ('zoomInDown', 'zoomInDown'),
        ('zoomInLeft', 'zoomInLeft'),
        ('zoomInRight', 'zoomInRight'),
        ('zoomInUp', 'zoomInUp'),
    )
    ),
    ('Specials', (
        ('hinge', 'hinge'),
        ('rollIn', 'rollIn'),
        ('rollOut', 'rollOut'),
    )
    ),
)


WIDGET_COLOR_SCHEME_CHOICES = (
    ('Brand styles', (
        ('default', 'Default'),
        ('primary', 'Primary'),
        ('secondary', 'Secondary'),
        ('custom', 'Custom'),
        ('red', 'Red'),
        ('blue', 'Blue'),
    )
    ),
)


PAGINATION_CHOICES = (
    ('paginator', 'Paginator'),
    ('cycle', 'Cycle'),
    ('slide', 'Slide'),
    ('grid', 'Grid'),
)




from datetime import datetime
import logging
try:
    from PIL import Image as PILImage
except ImportError:
    try:
        import Image as PILImage
    except ImportError:
        raise ImportError("The Python Imaging Library was not found.")

from django.conf import settings
from django.db import models
from django.utils.timezone import now, make_aware, get_current_timezone
from django.utils.translation import ugettext_lazy as _

from .abstract import BaseImage
from filer.utils.loader import load_object

logger = logging.getLogger("media")


class Image(BaseImage):
    date_taken = models.DateTimeField(_('date taken'), null=True, blank=True,
                                      editable=False)

    author = models.CharField(_('author'), max_length=255, null=True, blank=True)

    must_always_publish_author_credit = models.BooleanField(_('must always publish author credit'), default=False)
    must_always_publish_copyright = models.BooleanField(_('must always publish copyright'), default=False)

    filename_extensions = ['.jpg', '.jpeg', '.png', '.gif', ]

    def save(self, *args, **kwargs):
        if self.date_taken is None:
            try:
                exif_date = self.exif.get('DateTimeOriginal', None)
                if exif_date is not None:
                    d, t = exif_date.split(" ")
                    year, month, day = d.split(':')
                    hour, minute, second = t.split(':')
                    if getattr(settings, "USE_TZ", False):
                        tz = get_current_timezone()
                        self.date_taken = make_aware(datetime(
                            int(year), int(month), int(day),
                            int(hour), int(minute), int(second)), tz)
                    else:
                        self.date_taken = datetime(
                            int(year), int(month), int(day),
                            int(hour), int(minute), int(second))
            except Exception:
                pass
        if self.date_taken is None:
            self.date_taken = now()
        super(Image, self).save(*args, **kwargs)

    class Meta:
        verbose_name = _("image")
        verbose_name_plural = _('images')

        # You must define a meta with en explicit app_label
        app_label = 'media'




from django import http
from django.template import (Context, RequestContext,
                             loader, Template, TemplateDoesNotExist)
from django.views.decorators.csrf import requires_csrf_token
from django.template.loader import render_to_string

CONTENT_TYPE = 'text/html'


def _get_page_from_request(request):
    """return page from request
    """

    try:
        from leonardo.module.web.models import Page
        feincms_page = Page.objects.for_request(request, best_match=True)
    except:
        feincms_page = None

    return feincms_page


def render_in_page(request, template):

    """return rendered template in standalone mode or ``False``
    """

    page = _get_page_from_request(request)

    if page:
        try:
            slug = request.path_info.split("/")[-2:-1][0]
        except KeyError:
            slug = None

        try:
            body = render_to_string(template, RequestContext(request, {
                'request_path': request.path,
                'feincms_page': page,
                'slug': slug,
                'standalone': True}))
            response = http.HttpResponseNotFound(body, content_type=CONTENT_TYPE)
        except TemplateDoesNotExist:
            response = False

        return response

    return False


# This can be called when CsrfViewMiddleware.process_view has not run,
# therefore need @requires_csrf_token in case the template needs
# {% csrf_token %}.
@requires_csrf_token
def page_not_found(request, template_name='404.html'):
    """
    Default 404 handler.

    Templates: :template:`404.html`
    Context:
        request_path
            The path of the requested URL (e.g., '/app/pages/bad_page/')
    """
    response = render_in_page(request, template_name)

    if response:
        return response

    template = Template(
        '<h1>Not Found</h1>'
        '<p>The requested URL {{ request_path }} was not found on this server.</p>')
    body = template.render(RequestContext(request, {'request_path': request.path}))
    return http.HttpResponseNotFound(body, content_type=CONTENT_TYPE)


@requires_csrf_token
def server_error(request, template_name='500.html'):
    """
    500 error handler.

    Templates: :template:`500.html`
    Context: None
    """

    response = render_in_page(request, template_name)

    if response:
        return response

    try:
        template = loader.get_template(template_name)
    except TemplateDoesNotExist:
        return http.HttpResponseServerError('<h1>Server Error (500)</h1>', content_type='text/html')
    return http.HttpResponseServerError(template.render(Context({})))


@requires_csrf_token
def bad_request(request, template_name='400.html'):
    """
    400 error handler.

    Templates: :template:`400.html`
    Context: None
    """

    response = render_in_page(request, template_name)

    if response:
        return response

    try:
        template = loader.get_template(template_name)
    except TemplateDoesNotExist:
        return http.HttpResponseBadRequest('<h1>Bad Request (400)</h1>', content_type='text/html')
    return http.HttpResponseBadRequest(template.render(Context({})))


# This can be called when CsrfViewMiddleware.process_view has not run,
# therefore need @requires_csrf_token in case the template needs
# {% csrf_token %}.
@requires_csrf_token
def permission_denied(request, template_name='403.html'):
    """
    Permission denied (403) handler.

    Templates: :template:`403.html`
    Context: None

    If the template does not exist, an Http403 response containing the text
    "403 Forbidden" (as per RFC 2616) will be returned.
    """

    response = render_in_page(request, template_name)

    if response:
        return response

    try:
        template = loader.get_template(template_name)
    except TemplateDoesNotExist:
        return http.HttpResponseForbidden('<h1>403 Forbidden</h1>', content_type='text/html')
    return http.HttpResponseForbidden(template.render(RequestContext(request)))


def shortcut(request, content_type_id, object_id):
    from django.contrib.contenttypes.views import shortcut as real_shortcut
    return real_shortcut(request, content_type_id, object_id)




from django.apps import AppConfig

from .widget import *

LEONARDO_WIDGETS = [
    UserLoginWidget,
    UserRegistrationWidget
]

LEONARDO_APPS = ['leonardo.module.leonardo_auth']

default_app_config = 'leonardo.module.leonardo_auth.Config'

LEONARDO_OPTGROUP = 'Auth widgets'

LEONARDO_PAGE_ACTIONS = ['leonardo_auth/_actions.html']


class Config(AppConfig):
    name = 'leonardo.module.leonardo_auth'
    verbose_name = "Auth module"



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.template.context import RequestContext
from django.conf import settings
from leonardo.module.web.models import Widget


class SiteHeadingWidget(Widget):

    icon = "fa fa-text-width"

    site_title = models.CharField(
        max_length=255, verbose_name=_("Site Title"), null=True, blank=True)
    logo = models.ForeignKey('media.Image',
                             blank=True, null=True, verbose_name=_("Logo"), related_name="%(app_label)s_%(class)s_images")

    tagline = models.TextField(blank=True, verbose_name=_("Tagline"))

    def render_content(self, options):

        if not self.site_title:
            self.site_title = settings.META_TITLE

        context = RequestContext(options.get('request'), {
            'widget': self,
        })

        return render_to_string(self.get_template_name(), context)

    class Meta:
        abstract = True
        verbose_name = _("site heading")
        verbose_name_plural = _('site headings')




from django.utils import six
from django.conf import settings
import os
from optparse import make_option

from django.core.management.base import NoArgsCommand
from leonardo.module.web.models import Widget, WidgetContentTheme, WidgetBaseTheme

from ._utils import get_or_create_template

"widget.verbose_name - template.name"
THEME_NAME_FORMAT = "{0} {1}"


class Command(NoArgsCommand):
    help = "Syncs file system templates and themes with the database bidirectionally.\
    based on dbtemplates.sync_templates"
    option_list = NoArgsCommand.option_list + (
        make_option("-f", "--force",
                    action="store_true", dest="force", default=False,
                    help="overwrite existing database templates"),
    )

    def get_all_widget_classes(self):
        """returns collected Leonardo Widgets

        if not declared in settings is used __subclasses__
        which not supports widget subclassing

        """
        _widgets = getattr(settings,
                           'WIDGETS', Widget.__subclasses__())
        widgets = []
        if isinstance(_widgets, dict):
            for group, widget_cls in six.iteritems(_widgets):
                widgets.extend(widget_cls)
        elif isinstance(_widgets, list):
            widgets = _widgets
        return widgets

    def handle_noargs(self, **options):
        force = options.get('force')

        created_themes = 0
        synced_themes = 0

        # base
        path = os.path.dirname(os.path.abspath(__file__))
        possible_topdir = os.path.normpath(os.path.join(path,
                                                        os.pardir,
                                                        os.pardir,
                                                        "templates"))

        # load widget base templates
        widget_base_dir = os.path.join(possible_topdir, "base", "widget")

        for dirpath, subdirs, filenames in os.walk(widget_base_dir):
            for f in filenames:
                # ignore private members
                if not f.startswith("_"):
                    w_base_template = get_or_create_template(
                        f, force=force, prefix="base/widget")
                    synced_themes += 1
                    name = f.split("/")[-1].split(".")[0]
                    try:
                        widget_theme = WidgetBaseTheme.objects.get(
                            name__exact=name)
                    except WidgetBaseTheme.DoesNotExist:
                        widget_theme = WidgetBaseTheme()
                        widget_theme.name = name
                        widget_theme.label = name.split(".")[0].title()
                        widget_theme.template = w_base_template
                        widget_theme.save()
                        created_themes += 1

        # load widget templates and create widget themes with default base
        for w in self.get_all_widget_classes():
            templates = w.templates()
            for name in templates:
                # ignore private members
                if not name.startswith("_"):
                    widget_template = get_or_create_template(name, force=force)
                    synced_themes += 1

                    if not widget_template:
                        self.stdout.write('Template for "%s" not found' % name)
                        continue

                    try:
                        widget_theme = WidgetContentTheme.objects.get(
                            template__name__exact=name)
                    except WidgetContentTheme.DoesNotExist:
                        widget_theme = WidgetContentTheme()
                        widget_theme.name = name.split("/")[-1].split(".")[0]
                        widget_theme.label = THEME_NAME_FORMAT.format(
                            unicode(w._meta.verbose_name),
                            name.split("/")[-1].split(".")[0]).capitalize()
                        widget_theme.template = widget_template
                        widget_theme.widget_class = w.__name__
                        widget_theme.save()
                        created_themes += 1

        self.stdout.write('Successfully created %s widget themes' % created_themes)
        self.stdout.write('Successfully synced %s widget themes' % synced_themes)



from __future__ import unicode_literals

import django
from django import forms
from django.core import exceptions, validators
from django.db import models
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _


class MaxValueMultiFieldValidator(validators.MaxLengthValidator):
    clean = lambda self, x: len(','.join(x))
    code = 'max_multifield_value'


class MaxChoicesValidator(validators.MaxLengthValidator):
    message = _(u'You must select a maximum of  %(limit_value)d choices.')
    code = 'max_choices'


def get_max_length(choices, max_length, default=200):
    if max_length is None:
        if choices:
            return len(','.join([key for key, label in choices]))
        else:
            return default
    return max_length


class MultiSelectFormField(forms.MultipleChoiceField):
    widget = forms.CheckboxSelectMultiple

    def __init__(self, *args, **kwargs):
        self.max_choices = kwargs.pop('max_choices', None)
        self.max_length = kwargs.pop('max_length', None)
        super(MultiSelectFormField, self).__init__(*args, **kwargs)
        self.max_length = get_max_length(self.choices, self.max_length)
        self.validators.append(MaxValueMultiFieldValidator(self.max_length))
        if self.max_choices is not None:
            self.validators.append(MaxChoicesValidator(self.max_choices))


def add_metaclass(metaclass):
    """Class decorator for creating a class with a metaclass."""
    def wrapper(cls):
        orig_vars = cls.__dict__.copy()
        orig_vars.pop('__dict__', None)
        orig_vars.pop('__weakref__', None)
        for slots_var in orig_vars.get('__slots__', ()):
            orig_vars.pop(slots_var)
        return metaclass(cls.__name__, cls.__bases__, orig_vars)
    return wrapper


class MultiSelectField(models.CharField):

    """ Choice values can not contain commas. """

    def __init__(self, *args, **kwargs):
        self.max_choices = kwargs.pop('max_choices', None)
        super(MultiSelectField, self).__init__(*args, **kwargs)
        self.max_length = get_max_length(self.choices, self.max_length)
        self.validators[0] = MaxValueMultiFieldValidator(self.max_length)
        if self.max_choices is not None:
            self.validators.append(MaxChoicesValidator(self.max_choices))

    @property
    def flatchoices(self):
        return None

    def get_choices_default(self):
        return self.get_choices(include_blank=False)

    def get_choices_selected(self, arr_choices):
        choices_selected = []
        for choice_selected in arr_choices:
            choices_selected.append(choice_selected[0])
        return choices_selected

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_prep_value(value)

    def validate(self, value, model_instance):
        arr_choices = self.get_choices_selected(self.get_choices_default())
        for opt_select in value:
            if (opt_select not in arr_choices):
                if django.VERSION[0] == 1 and django.VERSION[1] >= 6:
                    raise exceptions.ValidationError(
                        self.error_messages['invalid_choice'] % {"value": value})
                else:
                    raise exceptions.ValidationError(
                        self.error_messages['invalid_choice'] % value)

    def get_default(self):
        default = super(MultiSelectField, self).get_default()
        if isinstance(default, int):
            default = str(default)
        return default

    def formfield(self, **kwargs):
        defaults = {'required': not self.blank,
                    'label': capfirst(self.verbose_name),
                    'help_text': self.help_text,
                    'choices': self.choices,
                    'max_length': self.max_length,
                    'max_choices': self.max_choices}
        if self.has_default():
            defaults['initial'] = self.get_default()
        defaults.update(kwargs)
        return MultiSelectFormField(**defaults)

    def get_prep_value(self, value):
        return '' if value is None else ",".join(value)

    def to_python(self, value):
        if value:
            return value if isinstance(value, list) else value.split(',')

    def contribute_to_class(self, cls, name):
        super(MultiSelectField, self).contribute_to_class(cls, name)
        if self.choices:
            def get_list(obj):
                fieldname = name
                choicedict = dict(self.choices)
                display = []
                if getattr(obj, fieldname):
                    for value in getattr(obj, fieldname):
                        item_display = choicedict.get(value, None)
                        if item_display is None:
                            try:
                                item_display = choicedict.get(int(value), value)
                            except (ValueError, TypeError):
                                item_display = value
                        display.append(str(item_display))
                return display

            def get_display(obj):
                return ", ".join(get_list(obj))

            setattr(cls, 'get_%s_list' % self.name, get_list)
            setattr(cls, 'get_%s_display' % self.name, get_display)

MultiSelectField = add_metaclass(models.SubfieldBase)(MultiSelectField)





class Default(object):

    core = ['web', 'nav', 'media', 'search', 'devel', 'leonardo_auth']

    @property
    def middlewares(self):
        MIDDLEWARE_CLASSES = [
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.middleware.http.ConditionalGetMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
            'django.middleware.locale.LocaleMiddleware',
            'django.contrib.sites.middleware.CurrentSiteMiddleware',

        ]
        import django
        if django.VERSION >= (1, 8, 0):
            MIDDLEWARE_CLASSES += [
                'django.contrib.auth.middleware.SessionAuthenticationMiddleware']
        else:
            MIDDLEWARE_CLASSES += ['django.middleware.doc.XViewMiddleware']

        try:
            import debug_toolbar
            MIDDLEWARE_CLASSES += ['debug_toolbar.middleware.DebugToolbarMiddleware']
        except ImportError:
            pass

        return MIDDLEWARE_CLASSES

    @property
    def apps(self):
        return [
            'django',

            'django_extensions',
            'django.contrib.contenttypes',
            'django.contrib.auth',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.staticfiles',
            'django.contrib.sitemaps',

            'rest_framework',
            'dbtemplates',

            'django_select2',

            'reversion',
            'compressor',

            'horizon',
            'horizon_contrib',

            'leonardo',

            'constance',
            'constance.backends.database',

        ]

    @property
    def context_processors(self):
        """return CORE Conent Type Processors
        """
        cp = [
            'django.contrib.auth.context_processors.auth',
            'horizon.context_processors.horizon',
            'django.contrib.messages.context_processors.messages',

            'constance.context_processors.config',
        ]
        import django

        if django.VERSION[:2] < (1, 8):

            cp += [
                'django.core.context_processors.debug',
                'django.core.context_processors.i18n',
                'django.core.context_processors.media',
                'django.core.context_processors.request',
                'django.core.context_processors.static',
            ]

        else:
            cp += [
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.request',
                'django.template.context_processors.static',
            ]

        return cp



from haystack import indexes, fields
from .models import File, Image, Document, Video


class FileIndex(indexes.SearchIndex, indexes.Indexable):

    """
    Index for File objects

    """

    text = fields.CharField(document=True, use_template=True)
    url = fields.CharField(model_attr="url")

    title = fields.CharField(model_attr="name", null=True)
    description = fields.CharField(model_attr="description", null=True)
    folder = fields.CharField(model_attr="folder", null=True)
    original_filename = fields.CharField(model_attr="original_filename", null=True)

    uploaded_at = fields.DateTimeField(model_attr='uploaded_at', null=True)

    def get_model(self):
        return File

    def index_queryset(self, using=None):
        return self.get_model().objects.filter(is_public=True)

    def get_updated_field(self):
        return "uploaded_at"


class ImageIndex(FileIndex):

    def get_model(self):
        return Image


class DocumentIndex(FileIndex):

    def get_model(self):
        return Document


class VideoIndex(FileIndex):

    def get_model(self):
        return Video




from leonardo.base import default

EMAIL = {
    'HOST': 'mail.domain.com',
    'PORT': '25',
    'USER': 'username',
    'PASSWORD': 'pwd',
    'SECURITY': True,
}

RAVEN_CONFIG = {}

ALLOWED_HOSTS = ['*']

USE_TZ = True

DEBUG = True

ADMINS = (
    ('admin', 'mail@leonardo.cz'),
)

DEFAULT_CHARSET = 'utf-8'

MANAGERS = ADMINS

SITE_ID = 1

SITE_NAME = 'Leonardo'

TIME_ZONE = 'Europe/Prague'

LANGUAGE_CODE = 'en'

LANGUAGES = (
    ('en', 'EN'),
    ('cs', 'CS'),
)

USE_I18N = True

DBTEMPLATES_USE_REVERSION = True

DBTEMPLATES_MEDIA_PREFIX = '/static-/'

DBTEMPLATES_USE_CODEMIRROR = False

DBTEMPLATES_USE_TINYMCE = False

DBTEMPLATES_AUTO_POPULATE_CONTENT = True

DBTEMPLATES_ADD_DEFAULT_SITE = True

FILER_ENABLE_PERMISSIONS = True  # noqa

MIDDLEWARE_CLASSES = default.middlewares

ROOT_URLCONF = 'leonardo.urls'

LEONARDO_BOOTSTRAP_URL = 'http://github.com/django-leonardo/django-leonardo/raw/develop/contrib/bootstrap/demo.yaml'

MARKITUP_FILTER = ('markitup.renderers.render_rest', {'safe_mode': True})

INSTALLED_APPS = default.apps

# For easy_thumbnails to support retina displays (recent MacBooks, iOS)

FEINCMS_USE_PAGE_ADMIN = False

LEONARDO_USE_PAGE_ADMIN = True

FEINCMS_DEFAULT_PAGE_MODEL = 'web.Page'

CONSTANCE_BACKEND = 'constance.backends.database.DatabaseBackend'

CONSTANCE_CONFIG = {}

# enable auto loading packages
LEONARDO_MODULE_AUTO_INCLUDE = True

# enable system module
LEONARDO_SYSTEM_MODULE = True

##########################


STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    'compressor.finders.CompressorFinder',
)

LOGIN_URL = '/'
LOGIN_REDIRECT_URL = '/'
LOGOUT_URL = "/"

LOGOUT_ON_GET = True

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'root': {
        'level': 'DEBUG',
        'handlers': ['console'],
    },
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'formatters': {
        'verbose': {
            'format': "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
            'datefmt': "%d/%b/%Y %H:%M:%S"
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'file': {
            'level': 'INFO',
            'class': 'logging.FileHandler',
            'filename': 'leonardo_app.log',
            'formatter': 'verbose'
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['console', 'file'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'leonardo': {
            'handlers': ['console', 'file'],
            'level': 'DEBUG',
            'propagate': True,
        },
    }
}

CRISPY_TEMPLATE_PACK = 'bootstrap3'

SECRET_KEY = None

APPS = []

PAGE_EXTENSIONS = []

MIGRATION_MODULES = {}

# use default leonardo auth urls
LEONARDO_AUTH = True

FEINCMS_TIDY_HTML = False

APPLICATION_CHOICES = []

ADD_JS_FILES = []

ADD_CSS_FILES = []

ADD_SCSS_FILES = []

ADD_JS_SPEC_FILES = []

ADD_ANGULAR_MODULES = []

ADD_PAGE_ACTIONS = []

ADD_WIDGET_ACTIONS = []

ADD_MIGRATION_MODULES = {}

CONSTANCE_CONFIG_GROUPS = {}

ABSOLUTE_URL_OVERRIDES = {}

SELECT2_CACHE_PREFIX = 'SELECT2'

MODULE_URLS = {}

WIDGETS = {}




from django import forms
from django.contrib.admin.util import unquote
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _

from .. import settings
from ..models import File
from ..views import (popup_param, popup_status, selectfolder_param,
                     selectfolder_status)
from .permissions import PrimitivePermissionAwareModelAdmin


class FileAdminChangeFrom(forms.ModelForm):
    class Meta:
        model = File
        exclude = ()


class FileAdmin(PrimitivePermissionAwareModelAdmin):
    list_display = ('label',)
    list_per_page = settings.MEDIA_PAGINATE_BY
    search_fields = ['name', 'original_filename', 'sha1', 'description']
    raw_id_fields = ('owner',)
    readonly_fields = ('sha1',)

    # save_as hack, because without save_as it is impossible to hide the
    # save_and_add_another if save_as is False. To show only save_and_continue
    # and save in the submit row we need save_as=True and in
    # render_change_form() override add and change to False.
    save_as = True

    form = FileAdminChangeFrom

    @classmethod
    def build_fieldsets(cls, extra_main_fields=(), extra_advanced_fields=(), extra_fieldsets=()):
        fieldsets = (
            (None, {
                'fields': ('name', 'owner', 'description',) + extra_main_fields,
            }),
            (_('Advanced'), {
                'fields': ('file', 'sha1',) + extra_advanced_fields,
                'classes': ('collapse',),
                }),
            ) + extra_fieldsets
        if settings.FILER_ENABLE_PERMISSIONS:
            fieldsets = fieldsets + (
                (None, {
                    'fields': ('is_public',)
                }),
            )
        return fieldsets

    def response_change(self, request, obj):
        """
        Overrides the default to be able to forward to the directory listing
        instead of the default change_list_view
        """
        r = super(FileAdmin, self).response_change(request, obj)
        if r['Location']:
            # it was a successful save
            if (r['Location'] in ['../'] or
                r['Location'] == self._get_post_url(obj)):
                # this means it was a save: redirect to the directory view
                if obj.folder:
                    url = reverse('admin:filer-directory_listing',
                                  kwargs={'folder_id': obj.folder.id})
                else:
                    url = reverse(
                            'admin:filer-directory_listing-unfiled_images')
                url = "%s%s%s" % (url,popup_param(request),
                                  selectfolder_param(request,"&"))
                return HttpResponseRedirect(url)
            else:
                # this means it probably was a save_and_continue_editing
                pass
        return r

    def render_change_form(self, request, context, add=False, change=False,
                           form_url='', obj=None):
        extra_context = {'show_delete': True,
                         'is_popup': popup_status(request),
                         'select_folder': selectfolder_status(request),}
        context.update(extra_context)
        return super(FileAdmin, self).render_change_form(
            request=request, context=context, add=False, change=False,
            form_url=form_url, obj=obj)

    def delete_view(self, request, object_id, extra_context=None):
        """
        Overrides the default to enable redirecting to the directory view after
        deletion of a image.

        we need to fetch the object and find out who the parent is
        before super, because super will delete the object and make it
        impossible to find out the parent folder to redirect to.
        """
        parent_folder = None
        try:
            obj = self.get_queryset(request).get(pk=unquote(object_id))
            parent_folder = obj.folder
        except self.model.DoesNotExist:
            obj = None

        r = super(FileAdmin, self).delete_view(
            request=request, object_id=object_id,
            extra_context=extra_context)

        url = r.get("Location", None)
        # Check against filer_file_changelist as file deletion is always made by
        # the base class
        if (url in ["../../../../", "../../"] or
                url == reverse("admin:filer_file_changelist") or
                url == reverse("admin:filer_image_changelist")):
            if parent_folder:
                url = reverse('admin:filer-directory_listing',
                              kwargs={'folder_id': parent_folder.id})
            else:
                url = reverse('admin:filer-directory_listing-unfiled_images')
            url = "%s%s%s" % (url,popup_param(request),
                              selectfolder_param(request,"&"))
            return HttpResponseRedirect(url)
        return r

    def get_model_perms(self, request):
        """
        It seems this is only used for the list view. NICE :-)
        """
        return {
            'add': True,
            'change': True,
            'delete': True,
        }

FileAdmin.fieldsets = FileAdmin.build_fieldsets()




from .simple_devices import simple_devices


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def add_client_type(request):
    # defaults
    request.is_android_device = False
    request.is_kindle_device = False
    request.is_ios_device = False
    request.is_ios5_device = False
    request.is_touch_device = False
    request.is_simple_device = False
    request.is_webkit = False
    request.is_webos = False
    request.is_wide_device = False
    request.is_windows_phone_device = False

    if 'HTTP_USER_AGENT' in request.META:
        s = request.META["HTTP_USER_AGENT"].lower()

        if 'applewebkit' in s:
            request.is_webkit = True

        if 'ipad' in s:
            request.is_ios_device = True
            request.is_touch_device = True
            request.is_wide_device = True

        elif 'iphone' in s or 'ipod' in s:
            request.is_ios_device = True
            request.is_touch_device = True

        elif 'android' in s:
            request.is_android_device = True
            request.is_touch_device = True

            if 'xoom' in s:
                request.is_wide_device = True

        elif 'webos' in s:
            request.is_webos_device = True
            request.is_touch_device = True

        elif 'windows phone' in s:
            request.is_windows_phone_device = True
            request.is_touch_device = True

        elif 'kindle' in s:
            request.is_kindle_device = True

        # Now that all the good devices are out of the way, lets see if this is an
        # old phone.

        elif 'HTTP_X_OPERAMINI_FEATURES' in request.META:
            request.is_simple_device = True

        elif 'application/vnd.wap.xhtml+xml' in request.META.get('HTTP_ACCEPT', '').lower():
            request.is_simple_device = True

        elif any([device in s for device in simple_devices]):
            request.is_simple_device = True

        else:
            # assume desktop at this point
            request.is_wide_device = True

        if request.is_ios_device and 'os 5' in s:
            request.is_ios5_device = True
    else:
        request.is_wide_device = True
    return request




from __future__ import absolute_import

from django.conf.urls import include, patterns, url

from .views import *


urlpatterns = patterns('',
                       url(r'^(?P<page_id>[\w\.\-]+)/update/$',
                           PageUpdateView.as_view(), name='page_update'),
                       url(r'^(?P<parent>[\w\.\-]+)/create/$',
                           PageCreateView.as_view(), name='page_create'),
                       url(r'^(?P<page_id>[\w\.\-]+)/delete/$',
                           PageDeleteView.as_view(), name='page_delete'),
                       url(r'^(?P<parent>[\w\.\-]+)/(?P<slug>[\w\.\-]+)/create/$',
                           PageCreateView.as_view(), name='page_create'),
                       url(r'^(?P<page_id>[\w\.\-]+)/dimension/add/$',
                           PageDimensionUpdateView.as_view(), name='page_dimension_add'),
                       url(r'^change/', include('leonardo.module.web.page.common.urls')),
                       )



# -#- coding: utf-8 -#-

from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo.module.nav.forms import NavigationForm
from leonardo.module.nav.models import NavigationWidget


class ContentNavigationWidget(NavigationWidget):
    include_contextual_pages = models.BooleanField(
        verbose_name=_("Display included pages"), default=False)
    include_text_headers = models.BooleanField(
        verbose_name=_("Display widget headers"), default=False)

    class Meta:
        abstract = True
        verbose_name = _("Content navigation")
        verbose_name_plural = _('Content navigations')

    feincms_item_editor_form = NavigationForm

    def render_content(self, options):
        request = options['request']
        page = request.webcms_page

        region = 'col3'
        headers = []
        contents = page.content.col3
        """
        for content in contents:
            if content.options.get('label', False):
                headers.append(content)
        """

        return render_to_string(self.get_template_name(), {
            'widget': self,
            'request': options['request'],
            'headers': headers,
            'page': page,
        })



from leonardo.module.web.widgets.forms import WidgetUpdateForm
from leonardo.module.web.page.widgets import PageSelectWidget
from leonardo.module.web.page.fields import PageSelectField


class NavigationForm(WidgetUpdateForm):

    root = PageSelectField(widget=PageSelectWidget(), required=False)



# -#- coding: utf-8 -#-

import re

from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.web.models import Widget

PORTALS = (
    ('youtube', re.compile(r'youtube'), lambda url: {
     'v': re.search(r'[?&]v=([^#&]+)', url).group(1)}),
    ('vimeo', re.compile(r'vimeo'), lambda url: {
     'id': re.search(r'/(\d+)', url).group(1)}),
    ('sf', re.compile(r'sf\.tv'), lambda url: {
     'id': re.search(r'/([a-z0-9\-]+)', url).group(1)}),
)


class InternetVideoWidget(Widget):

    """
    Copy-paste a URL to youtube or vimeo into the text box, this content type
    will automatically generate the necessary embed code.

    Other portals aren't supported currently, but would be easy to add if anyone
    would take up the baton.
    """

    icon = "fa fa-youtube-square"

    video = models.URLField(_('video link'),
                            help_text=_('This should be a link to a youtube or vimeo video, i.e.: http://www.youtube.com/watch?v=zmj1rpzDRZ0'))

    def source(self, **kwargs):
        for portal, match, context_fn in PORTALS:
            if match.search(self.video):
                return portal
        return 'unknown'

    def source_id(self, **kwargs):
        for portal, match, context_fn in PORTALS:
            if match.search(self.video):
                return context_fn(self.video)
        return self.video

    class Meta:
        abstract = True
        verbose_name = _('internet video')
        verbose_name_plural = _('internet videos')



# -#- coding: utf-8 -#-

from django import forms
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.forms.util import ErrorList
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
#from feincms.admin.item_editor import ItemEditorForm, FeinCMSInline
from feincms.utils import get_object
from leonardo.module.web.models import Widget
from leonardo.module.web.widgets.forms import WidgetUpdateForm


class HtmlTextWidgetAdminForm(WidgetUpdateForm):
    text = forms.CharField(
        widget=forms.Textarea(), max_length=9999, required=False, label=_('text'))

    class Meta:

        fields = ('text',)
        widgets = {'template_name': forms.widgets.Select(choices=[])}

    def __init__(self, *args, **kwargs):
        super(HtmlTextWidgetAdminForm, self).__init__(*args, **kwargs)
        self.fields['text'].widget.attrs.update({'class': 'item-richtext'})


    #: If FEINCMS_TIDY_ALLOW_WARNINGS_OVERRIDE allows, we'll convert this into
    # a checkbox so the user can choose whether to ignore HTML validation
    # warnings instead of fixing them:
    seen_tidy_warnings = forms.BooleanField(
        required=False,
        label=_("HTML Tidy"),
        help_text=_("Ignore the HTML validation warnings"),
        widget=forms.HiddenInput
    )

    def clean(self):
        cleaned_data = super(HtmlTextWidgetAdminForm, self).clean()

        if settings.FEINCMS_TIDY_HTML:
            text, errors, warnings = get_object(
                settings.FEINCMS_TIDY_FUNCTION)(cleaned_data['text'])

            # Ick, but we need to be able to update text and
            # seen_tidy_warnings:
            self.data = self.data.copy()

            # We always replace the HTML with the tidied version:
            cleaned_data['text'] = text
            self.data['%s-text' % self.prefix] = text

            if settings.FEINCMS_TIDY_SHOW_WARNINGS and (errors or warnings):
                if settings.FEINCMS_TIDY_ALLOW_WARNINGS_OVERRIDE:
                    # Convert the ignore input from hidden to Checkbox so the
                    # user can change it:
                    self.fields[
                        'seen_tidy_warnings'].widget = forms.CheckboxInput()

                if errors or not (settings.FEINCMS_TIDY_ALLOW_WARNINGS_OVERRIDE and cleaned_data['seen_tidy_warnings']):
                    self._errors["text"] = ErrorList([mark_safe(
                        _("HTML validation produced %(count)d warnings. Please review the updated content below before continuing: %(messages)s") % {
                            "count": len(warnings) + len(errors),
                            "messages": '<ul><li>%s</li></ul>' % "</li><li>".join(map(escape, errors + warnings))
                        }
                    )])

                # If we're allowed to ignore warnings and we don't have any
                # errors we'll set our hidden form field to allow the user to
                # ignore warnings on the next submit:
                if not errors and settings.FEINCMS_TIDY_ALLOW_WARNINGS_OVERRIDE:
                    self.data["%s-seen_tidy_warnings" % self.prefix] = True

        return cleaned_data


class HtmlTextWidget(Widget):
    form = HtmlTextWidgetAdminForm

    icon = "fa fa-file-text-o"

    #feincms_item_editor_inline = MyInline
    """
    feincms_item_editor_context_processors = (
        lambda x: settings.FEINCMS_RICHTEXT_INIT_CONTEXT,
    )
    feincms_item_editor_includes = {
        'head': ['admin/widget/htmltext/init_tinymce.html'],
    }
    """

    text = models.TextField(
        _('text'), blank=True, default="<p>%s</p>" % ('Empty element'))

    class Meta:
        abstract = True
        verbose_name = _('HTML text')
        verbose_name_plural = _('HTML texts')

    def save(self, *args, **kwargs):
        # TODO: Move this to the form?
        if getattr(self, 'cleanse', False):
            from feincms.utils.html.cleanse import cleanse_html
            self.text = cleanse_html(self.text)
        # self.rendered_content = self.render_content(kwargs)
        if self.text == '':
            self.text = "<p>%s</p>" % _('Empty element')
        super(HtmlTextWidget, self).save(*args, **kwargs)

    @classmethod
    def initialize_type(cls, cleanse=False):
        cls.cleanse = cleanse

        # TODO: Move this into somewhere more generic:
        if settings.FEINCMS_TIDY_HTML:
            # Make sure we can load the tidy function without dependency
            # failures:
            try:
                get_object(settings.FEINCMS_TIDY_FUNCTION)
            except ImportError as e:
                raise ImproperlyConfigured("FEINCMS_TIDY_HTML is enabled but the HTML tidy function %s could not be imported: %s" % (
                    settings.FEINCMS_TIDY_FUNCTION, e))



#-*- coding: utf-8 -*-
"""
Copy of ``django.contrib.admin.utils.get_deleted_objects`` and a subclass of
``django.contrib.admin.utils.NestedObjects`` that work with djongo_polymorphic querysets.
Ultimatly these should go directly into django_polymorphic or, in a more generic way, into django itself.

This code has been copied from Django 1.4.

At all locations where something has been changed, there are inline comments in the code.
"""
from __future__ import unicode_literals

from django.contrib.admin.util import NestedObjects, quote
from django.core.urlresolvers import reverse
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.text import capfirst
try:
    from django.utils.encoding import force_text
except ImportError:
    # Django < 1.5
    from django.utils.encoding import force_unicode as force_text

from filer.utils.compatibility import get_delete_permission


def get_deleted_objects(objs, opts, user, admin_site, using):
    """
    Find all objects related to ``objs`` that should also be deleted. ``objs``
    must be a homogenous iterable of objects (e.g. a QuerySet).

    Returns a nested list of strings suitable for display in the
    template with the ``unordered_list`` filter.

    """
    # --- begin patch ---
    collector = PolymorphicAwareNestedObjects(using=using)
    # --- end patch ---
    collector.collect(objs)
    perms_needed = set()

    def format_callback(obj):
        has_admin = obj.__class__ in admin_site._registry
        opts = obj._meta

        if has_admin:
            admin_url = reverse('%s:%s_%s_change'
                                % (admin_site.name,
                                   opts.app_label,
                                   opts.object_name.lower()),
                                None, (quote(obj._get_pk_val()),))
            p = get_delete_permission(opts)
            if not user.has_perm(p):
                perms_needed.add(opts.verbose_name)
                # Display a link to the admin page.
            return mark_safe('%s: <a href="%s">%s</a>' %
                             (escape(capfirst(opts.verbose_name)),
                              admin_url,
                              escape(obj)))
        else:
            # Don't display link to edit, because it either has no
            # admin or is edited inline.
            return '%s: %s' % (capfirst(opts.verbose_name),
                               force_text(obj))

    to_delete = collector.nested(format_callback)

    protected = [format_callback(obj) for obj in collector.protected]

    return to_delete, perms_needed, protected


class PolymorphicAwareNestedObjects(NestedObjects):

    def collect(self, objs, source_attr=None, **kwargs):
        if hasattr(objs, 'non_polymorphic'):
            # .filter() is needed, because there may already be cached polymorphic results in the queryset
            objs = objs.non_polymorphic().filter()
        return super(PolymorphicAwareNestedObjects, self).collect(objs, source_attr=source_attr, **kwargs)



#-*- coding: utf-8 -*-
import os
try:
    from PIL import Image as PILImage
except ImportError:
    try:
        import Image as PILImage
    except ImportError:
        raise ImportError("The Python Imaging Library was not found.")

import logging
logger = logging.getLogger(__name__)

from django.db import models
from django.utils import six
from django.utils.translation import ugettext_lazy as _

from .. import settings as filer_settings
from .filemodels import File
from filer.utils.filer_easy_thumbnails import FilerThumbnailer
from filer.utils.pil_exif import get_exif_for_file


class BaseImage(File):
    SIDEBAR_IMAGE_WIDTH = 210
    DEFAULT_THUMBNAILS = {
        'admin_clipboard_icon': {'size': (32, 32), 'crop': True,
                                 'upscale': True},
        'admin_sidebar_preview': {'size': (SIDEBAR_IMAGE_WIDTH, 10000)},
        'admin_directory_listing_icon': {'size': (48, 48),
                                         'crop': True, 'upscale': True},
        'admin_tiny_icon': {'size': (32, 32), 'crop': True, 'upscale': True},
    }
    file_type = 'Image'
    _icon = "image"

    _height = models.IntegerField(null=True, blank=True)
    _width = models.IntegerField(null=True, blank=True)

    default_alt_text = models.CharField(_('default alt text'), max_length=255, blank=True, null=True)
    default_caption = models.CharField(_('default caption'), max_length=255, blank=True, null=True)

    subject_location = models.CharField(_('subject location'), max_length=64, null=True, blank=True,
                                        default=None)

    @classmethod
    def matches_file_type(cls, iname, ifile=None, request=None):
        # This was originally in admin/clipboardadmin.py  it was inside of a try
        # except, I have moved it here outside of a try except because I can't
        # figure out just what kind of exception this could generate... all it was
        # doing for me was obscuring errors...
        # --Dave Butler <croepha@gmail.com>
        iext = os.path.splitext(iname)[1].lower()
        return iext in ['.jpg', '.jpeg', '.png', '.gif']

    def save(self, *args, **kwargs):
        self.has_all_mandatory_data = self._check_validity()
        try:
            # do this more efficient somehow?
            self.file.seek(0)
            self._width, self._height = PILImage.open(self.file).size
        except Exception:
            # probably the image is missing. nevermind.
            pass
        super(BaseImage, self).save(*args, **kwargs)

    def _check_validity(self):
        if not self.name:
            return False
        return True

    def sidebar_image_ratio(self):
        if self.width:
            return float(self.width) / float(self.SIDEBAR_IMAGE_WIDTH)
        else:
            return 1.0

    def _get_exif(self):
        if hasattr(self, '_exif_cache'):
            return self._exif_cache
        else:
            if self.file:
                self._exif_cache = get_exif_for_file(self.file.path)
            else:
                self._exif_cache = {}
        return self._exif_cache
    exif = property(_get_exif)

    def has_edit_permission(self, request):
        return self.has_generic_permission(request, 'edit')

    def has_read_permission(self, request):
        return self.has_generic_permission(request, 'read')

    def has_add_children_permission(self, request):
        return self.has_generic_permission(request, 'add_children')

    def has_generic_permission(self, request, permission_type):
        """
        Return true if the current user has permission on this
        image. Return the string 'ALL' if the user has all rights.
        """
        user = request.user
        if not user.is_authenticated():
            return False
        elif user.is_superuser:
            return True
        elif user == self.owner:
            return True
        elif self.folder:
            return self.folder.has_generic_permission(request, permission_type)
        else:
            return False

    @property
    def label(self):
        if self.name in ['', None]:
            return self.original_filename or 'unnamed file'
        else:
            return self.name

    @property
    def width(self):
        return self._width or 0

    @property
    def height(self):
        return self._height or 0

    def _generate_thumbnails(self, required_thumbnails):
        _thumbnails = {}
        for name, opts in six.iteritems(required_thumbnails):
            try:
                opts.update({'subject_location': self.subject_location})
                thumb = self.file.get_thumbnail(opts)
                _thumbnails[name] = thumb.url
            except Exception as e:
                # catch exception and manage it. We can re-raise it for debugging
                # purposes and/or just logging it, provided user configured
                # proper logging configuration
                if filer_settings.FILER_ENABLE_LOGGING:
                    logger.error('Error while generating thumbnail: %s',e)
                if filer_settings.FILER_DEBUG:
                    raise
        return _thumbnails

    @property
    def icons(self):
        required_thumbnails = dict(
            (size, {'size': (int(size), int(size)),
                    'crop': True,
                    'upscale': True,
                    'subject_location': self.subject_location})
            for size in filer_settings.FILER_ADMIN_ICON_SIZES)
        return self._generate_thumbnails(required_thumbnails)

    @property
    def thumbnails(self):
        return self._generate_thumbnails(BaseImage.DEFAULT_THUMBNAILS)

    @property
    def easy_thumbnails_thumbnailer(self):
        tn = FilerThumbnailer(
            file=self.file, name=self.file.name,
            source_storage=self.file.source_storage,
            thumbnail_storage=self.file.thumbnail_storage,
            thumbnail_basedir=self.file.thumbnail_basedir)
        return tn

    class Meta:
        app_label = 'media'
        verbose_name = _('image')
        verbose_name_plural = _('images')
        abstract = True





from importlib import import_module
from django.utils import six
from .versions import get_versions

import warnings

# define options
CONF_SPEC = {
    'optgroup': None,
    'urls_conf': None,
    'public': False,
    'plugins': [],
    'widgets': [],
    'apps': [],
    'middlewares': [],
    'context_processors': [],
    'dirs': [],
    'page_extensions': [],
    'auth_backends': [],
    'js_files': [],
    'js_spec_files': [],
    'angular_modules': [],
    'css_files': [],
    'scss_files': [],
    'config': {},
    'migration_modules': {},
    'absolute_url_overrides': {},
    'navigation_extensions': [],
    'page_actions': [],
    'widget_actions': [],
    'ordering': 0,
}

# just MAP - Django - Our spec
DJANGO_CONF = {
    'INSTALLED_APPS': "apps",
    'APPLICATION_CHOICES': "plugins",
    'MIDDLEWARE_CLASSES': "middlewares",
    'AUTHENTICATION_BACKENDS': "auth_backends",
    'PAGE_EXTENSIONS': "page_extensions",
    'ADD_JS_FILES': "js_files",
    'PAGE_EXTENSIONS': "page_extensions",
    'ADD_PAGE_ACTIONS': "page_actions",
    'ADD_WIDGET_ACTIONS': "widget_actions",
    'ADD_CSS_FILES': "css_files",
    'ADD_JS_FILES': "js_files",
    'ADD_JS_SPEC_FILES': "js_spec_files",
    'ADD_SCSS_FILES': "scss_files",
    'ADD_ANGULAR_MODULES': "angular_modules",
    'MIGRATION_MODULES': "migration_modules",
}

BLACKLIST = ['haystack']
LEONARDO_MODULES = None


def get_loaded_modules(modules):
    '''load modules and order it by ordering key'''

    _modules = []
    for mod in modules:
        mod_cfg = get_conf_from_module(mod)

        _modules.append((mod, mod_cfg,))

    _modules = sorted(_modules, key=lambda m: m[1].get('ordering'))

    return _modules


class dotdict(dict):

    """ Dictionary with dot access """

    def __getattr__(self, attr):
        return self.get(attr, None)
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__


class Config(dict):

    """Simple Module Config Object
    encapsulation of dot access dictionary

    use dictionary as constructor

    """

    def get_value(self, key, values):
        '''Accept key of propery and actual values'''
        return merge(values, self.get_property(key))

    def get_property(self, key):
        """Expect Django Conf property"""
        _key = DJANGO_CONF[key]
        return getattr(self, _key, CONF_SPEC[_key])

    @property
    def module_name(self):
        """Module name from module if is set"""
        if hasattr(self, "module"):
            return self.module.__name__
        return None

    @property
    def name(self):
        """Distribution name from module if is set"""
        if hasattr(self, "module"):
            return self.module.__name__.replace('_', '-')
        return None

    @property
    def version(self):
        """return module version"""
        return get_versions([self.module_name]).get(self.module_name, None)

    @property
    def latest_version(self):
        """return latest version if is available"""
        from leonardo_system.pip import check_versions
        return check_versions(True).get(self.name, None).get('new', None)

    @property
    def needs_migrations(self):
        """Indicates whater module needs migrations"""
        # TODO(majklk): also check models etc.
        if len(self.widgets) > 0:
            return True
        return False

    @property
    def needs_sync(self):
        """Indicates whater module needs templates, static etc."""

        affected_attributes = [
            'css_files', 'js_files',
            'scss_files', 'widgets']

        for attr in affected_attributes:
            if len(getattr(self, attr)) > 0:
                return True
        return False

    def set_module(self, module):
        """Just setter for module"""
        setattr(self, "module", module)

    def __getattr__(self, attr):
        return self.get(attr, None)

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__


def _get_key_from_module(mod, key, default):
    if hasattr(mod, key):
        value = getattr(mod, key, default)
    else:
        value = getattr(mod, 'LEONARDO_%s' % key.upper(), default)
    return value

CONFIG_VALID = (list, tuple, dict)


def merge(a, b):
    """return merged tuples or lists without duplicates
    note: ensure if admin theme is before admin
    """
    if isinstance(a, CONFIG_VALID) \
            and isinstance(b, CONFIG_VALID):
        # dict update
        if isinstance(a, dict) and isinstance(b, dict):
            a.update(b)
            return a
        # list update
        _a = list(a)
        for x in list(b):
            if x not in _a:
                _a.append(x)
        return _a
    if a and b:
        raise Exception("Cannot merge")
    raise NotImplementedError


def get_leonardo_modules():
    """return all leonardo modules

    check every installed module for leonardo descriptor

    by default use in memory cache
    """
    global LEONARDO_MODULES

    if not LEONARDO_MODULES:
        modules = []

        try:
            import pip
            installed_packages = pip.get_installed_distributions()
        except Exception:
            installed_packages = []
            warnings.warn(
                'pip is not installed module, scan module is skipped.',
                RuntimeWarning)

        for package in installed_packages:
            # check for default descriptor
            pkg_names = [k for k in package._get_metadata("top_level.txt")]
            for pkg_name in pkg_names:
                if pkg_name not in BLACKLIST:
                    try:
                        mod = import_module(pkg_name)
                        if hasattr(mod, 'default') \
                                or hasattr(mod, 'leonardo_module_conf'):
                            modules.append(mod)
                            continue
                        for key in dir(mod):
                            if 'LEONARDO' in key:
                                modules.append(mod)
                                break
                    except Exception:
                        pass
        LEONARDO_MODULES = modules
    return LEONARDO_MODULES


def extract_conf_from(mod, conf=Config(CONF_SPEC), depth=0, max_depth=2):
    """recursively extract keys from module or object
    by passed config scheme
    """

    # extract config keys from module or object
    for key, default_value in six.iteritems(conf):
        conf[key] = _get_key_from_module(mod, key, default_value)

    # support for recursive dependecies
    try:
        filtered_apps = [app for app in conf['apps'] if app not in BLACKLIST]
    except TypeError:
        pass
    except Exception as e:
        warnings.warn('Error %s during loading %s' % (e, conf['apps']))

    for app in filtered_apps:
        try:
            app_module = import_module(app)
            if app_module != mod:
                app_module = _get_correct_module(app_module)
                if depth < max_depth:
                    mod_conf = extract_conf_from(app_module, depth=depth+1)
                    for k, v in six.iteritems(mod_conf):
                        # prevent config duplicity
                        # skip config merge
                        if k == 'config':
                            continue
                        if isinstance(v, dict):
                            conf[k].update(v)
                        elif isinstance(v, (list, tuple)):
                            conf[k] = merge(conf[k], v)
        except Exception as e:
            pass  # swallow, but maybe log for info what happens
    return conf


def _get_correct_module(mod):
    """returns imported module
    check if is ``leonardo_module_conf`` specified and then import them
    """

    module_location = getattr(
        mod, 'leonardo_module_conf',
        getattr(mod, "LEONARDO_MODULE_CONF", None))
    if module_location:
        mod = import_module(module_location)
    return mod


def get_conf_from_module(mod):
    """return configuration from module with defaults no worry about None type

    """

    conf = Config(CONF_SPEC)

    # get imported module
    mod = _get_correct_module(mod)

    conf.set_module(mod)

    # extarct from default object or from module
    if hasattr(mod, 'default'):
        default = mod.default
        conf = extract_conf_from(default, conf)
    else:
        conf = extract_conf_from(mod, conf)
    return conf



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.media.fields import DocumentField
from leonardo.module.web.models import Widget
from leonardo.module.web.widgets.forms import WidgetUpdateForm


class DocumentForm(WidgetUpdateForm):

    file = DocumentField()


class PdfDocumentWidget(Widget):

    feincms_item_editor_form = DocumentForm

    icon = "fa fa-file-pdf-o"

    file = models.ForeignKey("media.Document", verbose_name=_(
        "pdf document"), related_name="%(app_label)s_%(class)s_related")
    height = models.CharField(
        max_length=255, verbose_name=_("height"), blank=True)
    width = models.CharField(
        max_length=255, verbose_name=_("width"), blank=True)

    class Meta:
        abstract = True
        verbose_name = _("pdf document")
        verbose_name_plural = _('pdf documents')



from __future__ import unicode_literals

import re

import types
from django.conf import settings
from django.core.urlresolvers import Resolver404, resolve
from django.http import HttpResponseNotFound
from django.template import RequestContext, Template
from django.template.loader import render_to_string
from django.utils.encoding import force_bytes
from django.views.debug import default_urlconf, get_safe_settings

HIDDEN_SETTINGS = re.compile('API|TOKEN|KEY|SECRET|PASS|SIGNATURE')

CLEANSED_SUBSTITUTE = '********************'

TECHNICAL_404_TEMPLATE = '404_technical.html'


def technical_404_response(request, exception):
    "Create a technical 404 error response. The exception should be the Http404."
    try:
        error_url = exception.args[0]['path']
    except (IndexError, TypeError, KeyError):
        error_url = request.path_info[1:]  # Trim leading slash

    try:
        tried = exception.args[0]['tried']
    except (IndexError, TypeError, KeyError):
        tried = []
    else:
        if (not tried                           # empty URLconf
            or (request.path == '/'
                and len(tried) == 1             # default URLconf
                and len(tried[0]) == 1
                and getattr(tried[0][0], 'app_name', '') == getattr(tried[0][0], 'namespace', '') == 'admin')):
            return default_urlconf(request)

    urlconf = getattr(request, 'urlconf', settings.ROOT_URLCONF)
    if isinstance(urlconf, types.ModuleType):
        urlconf = urlconf.__name__

    caller = ''
    try:
        resolver_match = resolve(request.path)
    except Resolver404:
        pass
    else:
        obj = resolver_match.func

        if hasattr(obj, '__name__'):
            caller = obj.__name__
        elif hasattr(obj, '__class__') and hasattr(obj.__class__, '__name__'):
            caller = obj.__class__.__name__

        if hasattr(obj, '__module__'):
            module = obj.__module__
            caller = '%s.%s' % (module, caller)

    try:
        from leonardo.module.web.models import Page
        feincms_page = Page.objects.for_request(request, best_match=True)
        template = feincms_page.theme.template
    except:
        feincms_page = None
        slug = None
        template = None
    else:
        # nested path is not allowed for this time
        try:
            slug = request.path_info.split("/")[-2:-1][0]
        except KeyError:
            raise Exception("Nested path is not allowed !")

    c = RequestContext(request, {
        'urlconf': urlconf,
        'root_urlconf': settings.ROOT_URLCONF,
        'request_path': error_url,
        'urlpatterns': tried,
        'reason': force_bytes(exception, errors='replace'),
        'request': request,
        'settings': get_safe_settings(),
        'raising_view_name': caller,
        'feincms_page': feincms_page,
        'template': template or 'base.html',
        'standalone': True,
        'slug': slug,
    })

    try:
        t = render_to_string('404_technical.html', c)
    except:
        from django.views.debug import TECHNICAL_404_TEMPLATE
        t = Template(TECHNICAL_404_TEMPLATE).render(c)
    return HttpResponseNotFound(t, content_type='text/html')



# -#- coding: utf-8 -#-

from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo.module.nav.forms import NavigationForm
from leonardo.module.nav.models import NavigationWidget
from leonardo.module.web.models import Page

DEPTH_CHOICES = (
    (1, _("Only one level")),
    (2, _("Include subpages")),
)
LINK_CHOICES = (
    ('default', _('Default title')),
    ('detail', _('Detailed title')),
)


class TreeNavigationWidget(NavigationWidget):
    depth = models.IntegerField(verbose_name=_("depth"), choices=DEPTH_CHOICES, default=1)
    root = models.ForeignKey(Page, blank=True, null=True, verbose_name=_(
        "root page"), related_name="taxonomy_root", help_text=_("If not set, widget's parent page will be used as root page."))
    link_style = models.CharField(
        max_length=255, verbose_name=_("Link style"), choices=LINK_CHOICES, default='default')

    class Meta:
        abstract = True
        verbose_name = _("Navigation menu")
        verbose_name_plural = _('Navigation menus')

    feincms_item_editor_form = NavigationForm

    def level(self, level):
        return level + 2

    def render_content(self, options):
        request = options['request']

        if self.root:
            root = self.root
        else:
            root = self.parent

        return render_to_string(self.get_template_name(), {
            'widget': self,
            'current': getattr(request, 'leonardo_page', None),
            'root': root,
            'level': self.level(root.level),
            'level2': self.level(root.level) + 1,
            'depth': self.depth,
            'request': request,
        })




import json
from django.conf import settings
from django.forms.models import modelform_factory
from django.contrib import admin
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from .. import settings as media_settings
from ..models import Clipboard, ClipboardItem
from filer.utils.files import handle_upload, UploadException
from filer.utils.loader import load_object


# ModelAdmins
class ClipboardItemInline(admin.TabularInline):
    model = ClipboardItem


class ClipboardAdmin(admin.ModelAdmin):
    model = Clipboard
    inlines = [ClipboardItemInline]
    filter_horizontal = ('files',)
    raw_id_fields = ('user',)
    verbose_name = "DEBUG Clipboard"
    verbose_name_plural = "DEBUG Clipboards"

    def get_urls(self):
        from django.conf.urls import patterns, url
        urls = super(ClipboardAdmin, self).get_urls()
        from .. import views
        url_patterns = patterns('',
            url(r'^operations/paste_clipboard_to_folder/$',
                self.admin_site.admin_view(views.paste_clipboard_to_folder),
                name='filer-paste_clipboard_to_folder'),
            url(r'^operations/discard_clipboard/$',
                self.admin_site.admin_view(views.discard_clipboard),
                name='filer-discard_clipboard'),
            url(r'^operations/delete_clipboard/$',
                self.admin_site.admin_view(views.delete_clipboard),
                name='filer-delete_clipboard'),
            # upload does it's own permission stuff (because of the stupid
            # flash missing cookie stuff)
            url(r'^operations/upload/$',
                self.ajax_upload,
                name='filer-ajax_upload'),
        )
        url_patterns.extend(urls)
        return url_patterns

    @csrf_exempt
    def ajax_upload(self, request, folder_id=None):
        """
        receives an upload from the uploader. Receives only one file at the time.
        """
        mimetype = "application/json" if request.is_ajax() else "text/html"
        content_type_key = 'content_type'
        response_params = {content_type_key: mimetype}
        try:
            upload, filename, is_raw = handle_upload(request)

            # Get clipboad
            clipboard = Clipboard.objects.get_or_create(user=request.user)[0]

            # find the file type
            for filer_class in media_settings.MEDIA_FILE_MODELS:
                FileSubClass = load_object(filer_class)
                #TODO: What if there are more than one that qualify?
                if FileSubClass.matches_file_type(filename, upload, request):
                    FileForm = modelform_factory(
                        model=FileSubClass,
                        fields=('original_filename', 'owner', 'file')
                    )
                    break
            uploadform = FileForm({'original_filename': filename,
                                   'owner': request.user.pk},
                                  {'file': upload})
            if uploadform.is_valid():
                file_obj = uploadform.save(commit=False)
                # Enforce the FILER_IS_PUBLIC_DEFAULT
                file_obj.is_public = settings.MEDIA_IS_PUBLIC_DEFAULT
                file_obj.save()
                clipboard_item = ClipboardItem(
                    clipboard=clipboard, file=file_obj)
                clipboard_item.save()
                json_response = {
                    'thumbnail': file_obj.icons['32'],
                    'alt_text': '',
                    'label': str(file_obj),
                }
                return HttpResponse(json.dumps(json_response),
                                    **response_params)
            else:
                form_errors = '; '.join(['%s: %s' % (
                    field,
                    ', '.join(errors)) for field, errors in list(uploadform.errors.items())
                ])
                raise UploadException("AJAX request not valid: form invalid '%s'" % (form_errors,))
        except UploadException as e:
            return HttpResponse(json.dumps({'error': str(e)}),
                                **response_params)

    def get_model_perms(self, request):
        """
        It seems this is only used for the list view. NICE :-)
        """
        return {
            'add': False,
            'change': False,
            'delete': False,
        }




from django import forms
from django.utils.translation import ugettext as _

from .. import settings as filer_settings
from ..models import Image
from .fileadmin import FileAdmin


class ImageAdminForm(forms.ModelForm):
    subject_location = forms.CharField(
        max_length=64, required=False,
        label=_('Subject location'),
        help_text=_('Location of the main subject of the scene.'))

    def sidebar_image_ratio(self):
        if self.instance:
            # this is very important. It forces the value to be returned as a
            # string and always with a "." as seperator. If the conversion
            # from float to string is done in the template, the locale will
            # be used and in some cases there would be a "," instead of ".".
            # javascript would parse that to an integer.
            return '%.6F' % self.instance.sidebar_image_ratio()
        else:
            return ''

    class Meta:
        model = Image
        exclude = ()

    class Media:
        css = {
            #'all': (settings.MEDIA_URL + 'filer/css/focal_point.css',)
        }
        js = (
            filer_settings.FILER_STATICMEDIA_PREFIX + 'js/raphael.js',
            filer_settings.FILER_STATICMEDIA_PREFIX + 'js/focal_point.js',
        )


class ImageAdmin(FileAdmin):
    form = ImageAdminForm


ImageAdmin.fieldsets = ImageAdmin.build_fieldsets(
    extra_main_fields=('author', 'default_alt_text', 'default_caption',),
    extra_fieldsets=(
        ('Subject Location', {
            'fields': ('subject_location',),
            'classes': ('collapse',),
        }),
    )
)




import pprint
import logging
from django.utils import six
try:
    import pkg_resources
except ImportError:
    pkg_resources = None  # NOQA
import sys

logger = logging.getLogger('leonardo.system')

pp = pprint.PrettyPrinter(indent=4)


def varmap(func, var, context=None, name=None):
    """
    Executes ``func(key_name, value)`` on all values
    recurisively discovering dict and list scoped
    values.
    """
    if context is None:
        context = {}
    objid = id(var)
    if objid in context:
        return func(name, '<...>')
    context[objid] = 1
    if isinstance(var, dict):
        ret = dict((k, varmap(func, v, context, k)) for k, v in six.iteritems(var))
    elif isinstance(var, (list, tuple)):
        ret = [varmap(func, f, context, name) for f in var]
    else:
        ret = func(name, var)
    del context[objid]
    return ret

# We store a cache of module_name->version string to avoid
# continuous imports and lookups of modules
_VERSION_CACHE = {}


def get_version_from_app(module_name, app):
    version = None
    if hasattr(app, 'get_version'):
        version = app.get_version
    elif hasattr(app, '__version__'):
        version = app.__version__
    elif hasattr(app, 'VERSION'):
        version = app.VERSION
    elif hasattr(app, 'version'):
        version = app.version

    if callable(version):
        version = version()

    if not isinstance(version, (six.string_types, list, tuple)):
        version = None

    if version is None:
        if pkg_resources is None:
            return None
        # pull version from pkg_resources if distro exists
        try:
            version = pkg_resources.get_distribution(module_name).version
        except pkg_resources.DistributionNotFound:
            return None

    if isinstance(version, (list, tuple)):
        version = '.'.join(map(str, version))

    return str(version)


def get_versions(module_list=None):
    if not module_list:
        return {}

    ext_module_list = set()
    for m in module_list:
        parts = m.split('.')
        ext_module_list.update('.'.join(parts[:idx]) for idx in range(1, len(parts) + 1))

    versions = {}
    for module_name in ext_module_list:
        if module_name not in _VERSION_CACHE:
            try:
                __import__(module_name)
            except ImportError:
                continue

            try:
                app = sys.modules[module_name]
            except KeyError:
                continue

            try:
                version = get_version_from_app(module_name, app)
            except Exception as e:
                logger.exception(e)
                version = None

            _VERSION_CACHE[module_name] = version
        else:
            version = _VERSION_CACHE[module_name]
        if version is None:
            continue
        versions[module_name] = version
    return versions




from __future__ import absolute_import

HORIZON_CONFIG = {
    # Allow for ordering dashboards; list or tuple if provided.
    'dashboards': [],

    # Name of a default dashboard; defaults to first alphabetically if None
    'default_dashboard': "",

    # Default redirect url for users' home
    'user_home': "",


    # URL for additional help with this site.
    'help_url': None,

    # Exception configuration.
    'exceptions': {'unauthorized': [],
                   'not_found': [],
                   'recoverable': []},

    # Password configuration.
    'password_validator': {'regex': '.*',
                           'help_text': ("Password is not accepted")},

    'password_autocomplete': 'on',

    # AJAX settings for JavaScript
    'ajax_queue_limit': 10,

    'ajax_poll_interval': 2500,

    'auto_fade_alerts': {
        'delay': 3000,
        'fade_duration': 1500,
        'types': ['alert-success', 'alert-info']
    },

    'angular_modules': [],
    'js_files': [],
    'js_spec_files': [],

}



# -#- coding: utf-8 -#-

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.web.models import Widget
from leonardo.module.media.fields.image import ImageField

from leonardo.module.web.widgets.forms import WidgetUpdateForm

ON_CLICK_CHOICES = (
    ('open_modal', _('open in modal window')),
    ('open_new_window', _('open new window')),
    ('on_page', _('disply in page')),
)

SIZE_CHOICES = (
    ('small', _('small')),
    ('medium', _('medium')),
    ('large', _('large')),
)


class ImageForm(WidgetUpdateForm):

    image = ImageField()


class SimpleImageWidget(Widget):

    icon = "fa fa-file-image-o"

    feincms_item_editor_form = ImageForm

    image = models.ForeignKey(
        "media.Image", verbose_name=_("image"), related_name="%(app_label)s_%(class)s_images")
    size = models.CharField(
        max_length=255, verbose_name=_("size"), choices=SIZE_CHOICES, blank=True)
    detail = models.CharField(max_length=255, verbose_name=_(
        "on click action"), choices=ON_CLICK_CHOICES, blank=True)

    def get_size(self):
        return getattr(settings,
                       'MEDIA_THUMB_%s_GEOM' % self.size.upper(),
                       '96x96')

    class Meta:
        abstract = True
        verbose_name = _("simple image")
        verbose_name_plural = _('simple images')



from ..settings import FILER_ADMIN_ICON_SIZES, FILER_STATICMEDIA_PREFIX


class IconsMixin(object):
    """
    Can be used on any model that has a _icon attribute. will return a dict
    containing urls for icons of different sizes with that name.
    """
    @property
    def icons(self):
        r = {}
        if getattr(self, '_icon', False):
            for size in FILER_ADMIN_ICON_SIZES:
                r[size] = "%sicons/%s_%sx%s.png" % (
                            FILER_STATICMEDIA_PREFIX, self._icon, size, size)
        return r




from django_select2.forms import ModelSelect2Widget
from leonardo.models import Page, PageColorScheme, PageTheme


class PageSelectWidget(ModelSelect2Widget):

    model = Page

    search_fields = ['title__icontains', ]

    def label_from_instance(self, obj):
        """
        Coerces ``value`` to a Python data type.
        """
        return obj.tree_label


class PageColorSchemeSelectWidget(ModelSelect2Widget):

    model = PageColorScheme

    search_fields = ['label__icontains', 'name__icontains']


class PageThemeSelectWidget(ModelSelect2Widget):

    model = PageTheme

    search_fields = ['label__icontains', 'name__icontains']




from django.conf.urls import patterns, url
from django.views.generic import RedirectView

from . import views

urlpatterns = patterns(
    "",
    url(r"^login/$", views.LoginView.as_view(), name="login"),
    url(r"^signup/$", views.SignupView.as_view(), name="signup"),
    url(r"^logout/$", views.LogoutView.as_view(), name="logout"),
)

"""
    url(r"^signup/$", views.SignupView.as_view(), name="signup"),
url(r"^logout/$", views.logout, name="account_logout"),

url(r"^password/change/$", views.password_change,
    name="account_change_password"),
url(r"^password/set/$", views.password_set, name="account_set_password"),

url(r"^inactive/$", views.account_inactive, name="account_inactive"),

# E-mail
url(r"^email/$", views.email, name="account_email"),
url(r"^confirm-email/$", views.email_verification_sent,
    name="account_email_verification_sent"),
url(r"^confirm-email/(?P<key>\w+)/$", views.confirm_email,
    name="account_confirm_email"),
# Handle old redirects
url(r"^confirm_email/(?P<key>\w+)/$",
    RedirectView.as_view(url='/accounts/confirm-email/%(key)s/',
                         permanent=True)),

# password reset
url(r"^password/reset/$", views.password_reset,
    name="account_reset_password"),
url(r"^password/reset/done/$", views.password_reset_done,
    name="account_reset_password_done"),
url(r"^password/reset/key/(?P<uidb36>[0-9A-Za-z]+)-(?P<key>.+)/$",
    views.password_reset_from_key,
    name="account_reset_password_from_key"),
url(r"^password/reset/key/done/$", views.password_reset_from_key_done,
    name="account_reset_password_from_key_done"),
"""



COMPRESS_PRECOMPILERS = (
    ('text/scss', 'django_pyscss.compressor.DjangoScssFilter'),
)

COMPRESS_CSS_FILTERS = (
    'compressor.filters.css_default.CssAbsoluteFilter',
)

COMPRESS_ENABLED = True
COMPRESS_OFFLINE = False
COMPRESS_OUTPUT_DIR = 'dashboard'
COMPRESS_CSS_HASHING_METHOD = 'hash'
COMPRESS_PARSER = 'compressor.parser.HtmlParser'

import xstatic.main
import xstatic.pkg.angular
import xstatic.pkg.angular_bootstrap
import xstatic.pkg.angular_lrdragndrop
import xstatic.pkg.angular_smart_table
import xstatic.pkg.bootstrap_datepicker
import xstatic.pkg.bootstrap_scss
import xstatic.pkg.d3
import xstatic.pkg.font_awesome
import xstatic.pkg.hogan
import xstatic.pkg.jasmine
import xstatic.pkg.jquery
import xstatic.pkg.jquery_migrate
import xstatic.pkg.jquery_quicksearch
import xstatic.pkg.jquery_tablesorter
import xstatic.pkg.jquery_ui
import xstatic.pkg.jsencrypt
import xstatic.pkg.magic_search
import xstatic.pkg.qunit
import xstatic.pkg.rickshaw
import xstatic.pkg.spin
import xstatic.pkg.termjs
import xstatic.pkg.adminlte
import xstatic.pkg.react

webroot='/'

STATICFILES_DIRS = [
    ('horizon/lib/angular',
        xstatic.main.XStatic(xstatic.pkg.angular,
                             root_url=webroot).base_dir),
    ('horizon/lib/angular',
        xstatic.main.XStatic(xstatic.pkg.angular_bootstrap,
                             root_url=webroot).base_dir),
    ('horizon/lib/angular',
        xstatic.main.XStatic(xstatic.pkg.angular_lrdragndrop,
                             root_url=webroot).base_dir),
    ('horizon/lib/angular',
        xstatic.main.XStatic(xstatic.pkg.angular_smart_table,
                             root_url=webroot).base_dir),
    ('horizon/lib/bootstrap_datepicker',
        xstatic.main.XStatic(xstatic.pkg.bootstrap_datepicker,
                             root_url=webroot).base_dir),
    ('bootstrap',
        xstatic.main.XStatic(xstatic.pkg.bootstrap_scss,
                             root_url=webroot).base_dir),
    ('horizon/lib',
        xstatic.main.XStatic(xstatic.pkg.d3,
                             root_url=webroot).base_dir),
    ('horizon/lib',
        xstatic.main.XStatic(xstatic.pkg.hogan,
                             root_url=webroot).base_dir),
    ('horizon/lib/font-awesome',
        xstatic.main.XStatic(xstatic.pkg.font_awesome,
                             root_url=webroot).base_dir),
    ('horizon/lib/jasmine',
        xstatic.main.XStatic(xstatic.pkg.jasmine,
                             root_url=webroot).base_dir),
    ('horizon/lib/jquery',
        xstatic.main.XStatic(xstatic.pkg.jquery,
                             root_url=webroot).base_dir),
    ('horizon/lib/jquery',
        xstatic.main.XStatic(xstatic.pkg.jquery_migrate,
                             root_url=webroot).base_dir),
    ('horizon/lib/jquery',
        xstatic.main.XStatic(xstatic.pkg.jquery_quicksearch,
                             root_url=webroot).base_dir),
    ('horizon/lib/jquery',
        xstatic.main.XStatic(xstatic.pkg.jquery_tablesorter,
                             root_url=webroot).base_dir),
    ('horizon/lib/jsencrypt',
        xstatic.main.XStatic(xstatic.pkg.jsencrypt,
                             root_url=webroot).base_dir),
    ('horizon/lib/magic_search',
        xstatic.main.XStatic(xstatic.pkg.magic_search,
                             root_url=webroot).base_dir),
    ('horizon/lib/qunit',
        xstatic.main.XStatic(xstatic.pkg.qunit,
                             root_url=webroot).base_dir),
    ('horizon/lib',
        xstatic.main.XStatic(xstatic.pkg.rickshaw,
                             root_url=webroot).base_dir),
    ('horizon/lib',
        xstatic.main.XStatic(xstatic.pkg.spin,
                             root_url=webroot).base_dir),
    ('horizon/lib',
        xstatic.main.XStatic(xstatic.pkg.termjs,
                             root_url=webroot).base_dir),
    ('lib',
        xstatic.main.XStatic(xstatic.pkg.adminlte).base_dir),
    ('lib',
        xstatic.main.XStatic(xstatic.pkg.react).base_dir),
]

if xstatic.main.XStatic(xstatic.pkg.jquery_ui).version.startswith('1.10.'):
    # The 1.10.x versions already contain the 'ui' directory.
    STATICFILES_DIRS.append(('horizon/lib/jquery-ui',
        xstatic.main.XStatic(xstatic.pkg.jquery_ui).base_dir))
else:
    # Newer versions dropped the directory, add it to keep the path the same.
    STATICFILES_DIRS.append(('horizon/lib/jquery-ui/ui',
        xstatic.main.XStatic(xstatic.pkg.jquery_ui).base_dir))



# -#- coding: utf-8 -#-

from django.utils.translation import ugettext_lazy as _
from leonardo.module.web.models import Widget
from markupfield.fields import MarkupField


class MarkupTextWidget(Widget):
    text = MarkupField(_('text'), blank=True, default=_(
        'Empty text'), default_markup_type='restructuredtext')

    class Meta:
        abstract = True
        verbose_name = _('markup text')
        verbose_name_plural = _('markup texts')



# -#- coding: utf-8 -#-

from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo.module.nav.models import NavigationWidget


def get_page_url(page):
    if page.redirect_to:
        return page.redirect_to
    else:
        return page.get_absolute_url()

class BreadcrumbsWidget(NavigationWidget):
    root_text = models.CharField(
        max_length=255, verbose_name=_("Root node label"), blank=True, null=True)

    class Meta:
        abstract = True
        verbose_name = _("Breadcrumb trail")
        verbose_name_plural = _('Breadcrumb trails')

    def render_content(self, options):
        request = options['request']
        page = request.leonardo_page
        include_self = False
        extension = ''

        try:
            fragments = request._feincms_fragments
        except:
            fragments = {}

        if '_breadcrumbs' in fragments:
            extension = fragments["_breadcrumbs"].strip()
            if extension == '':
                include_self = True
        else:
            include_self = True

        ancs = page.get_ancestors()
        bc = []
        for anc in ancs:
            if anc.is_root_node():
                bc.append(
                    (get_page_url(anc.get_children()[0]), anc.in_navigation, anc.short_title()))
            else:
                bc.append((get_page_url(anc), anc.in_navigation, anc.short_title()))

        if include_self:
            bc.append(('', True, page.short_title()))

        return render_to_string(self.get_template_name(), {
            'widget': self,
            'page': page,
            "trail": bc,
            "extension": extension,
            "include_self": include_self,
            'request': options['request'],
        })




import copy

import floppyforms as forms
from crispy_forms.bootstrap import Tab, TabHolder
from crispy_forms.layout import Field, HTML, Layout, Fieldset
from django.forms.models import modelform_factory
from django.utils.translation import ugettext_lazy as _
from horizon.utils.memoized import memoized
from django import forms as django_forms
from horizon_contrib.common import get_class
from leonardo.forms import SelfHandlingModelForm, SelfHandlingForm
from django.utils.text import slugify
from ..models import Page, PageTheme, PageColorScheme


class SwitchableFormFieldMixin(object):

    def get_switched_form_field_attrs(self, prefix, input_type, name):
        """Creates attribute dicts for the switchable theme form
        """
        attributes = {'class': 'switched', 'data-switch-on': prefix + 'field'}
        attributes['data-' + prefix + 'field-' + input_type] = name
        return attributes

    def switchable_field_attrs(self):
        return {'class': 'switchable',
                'data-slug': 'switchablefield'
                }


class PageColorSchemeSwitchableFormMixin(SwitchableFormFieldMixin):

    def init_color_scheme_switch(self, color_scheme=None):
        color_scheme_fields = []

        for theme in self.fields['theme'].queryset:

            name = 'theme__%s' % theme.id
            attributes = self.get_switched_form_field_attrs(
                'switchable', '%s' % theme.id, ('Color Scheme'))
            field = django_forms.ModelChoiceField(label=_('Color Scheme'),
                                                  queryset=theme.templates.all(),
                                                  required=False)
            # inital for color scheme
            if color_scheme and theme.templates.filter(id=color_scheme.id).exists():
                field.initial = color_scheme
            elif 'parent' in self.fields and self.fields['parent'].initial:
                field.initial = self.fields['parent'].initial.color_scheme
            elif self.instance and hasattr(self.instance, 'color_scheme'):
                field.initial = self.instance.color_scheme
            else:
                field.initial = theme.templates.first()
            self.fields[name] = field
            color_scheme_fields.append(name)

        # update theme widget attributes
        self.fields['theme'].widget.attrs = self.switchable_field_attrs()
        return color_scheme_fields


class PageCreateForm(PageColorSchemeSwitchableFormMixin, SelfHandlingModelForm):

    slug = forms.SlugField(required=False, initial=None)

    class Meta:
        model = Page
        widgets = {
            'parent': forms.widgets.HiddenInput,
        }
        exclude = tuple()

    def clean_slug(self):
        """slug title if is not provided
        """
        slug = self.cleaned_data.get('slug', None)
        if slug is None or len(slug) == 0:
            slug = slugify(self.cleaned_data['title'])
        return slug

    def __init__(self, *args, **kwargs):
        parent = kwargs.pop('parent', None)
        super(PageCreateForm, self).__init__(*args, **kwargs)

        self.fields['parent'].initial = parent
        color_scheme_fields = self.init_color_scheme_switch(
            color_scheme=kwargs['initial'].get('color_scheme', None))

        self.helper.layout = Layout(
            TabHolder(
                Tab(_('Main'),
                    'title',
                    'language',
                    'translation_of',
                    'site',
                    css_id='page-main'
                    ),
                Tab(_('Navigation'),
                    'in_navigation', 'parent', 'slug', 'override_url', 'redirect_to',
                    'symlinked_page'
                    ),
                Tab(_('Heading'),
                    '_content_title', '_page_title',
                    css_id='page-heading'
                    ),
                Tab(_('Publication'),
                    'active', 'featured', 'publication_date', 'publication_end_date',
                    ),
                Tab(_('Theme'),
                    'template_key', 'layout', Fieldset(
                        'Themes', 'theme', *color_scheme_fields),
                    css_id='page-theme-settings'
                    ),
            )
        )

        self.fields['color_scheme'].required = False

    def clean(self):
        cleaned = super(PageCreateForm, self).clean()
        theme = cleaned['theme']
        cleaned['color_scheme'] = self.cleaned_data['theme__%s' % theme.id]
        return cleaned


class PageUpdateForm(PageColorSchemeSwitchableFormMixin, SelfHandlingModelForm):

    class Meta:
        model = Page
        widgets = {
            'parent': forms.widgets.HiddenInput,
            'override_url': forms.widgets.HiddenInput,
            'publication_date': forms.widgets.DateInput,
        }
        exclude = tuple()

    def clean(self):
        cleaned = super(PageUpdateForm, self).clean()
        theme = cleaned['theme']
        cleaned['color_scheme'] = self.cleaned_data['theme__%s' % theme.id]
        return cleaned

    def __init__(self, *args, **kwargs):
        request = kwargs.pop('request', None)
        super(PageUpdateForm, self).__init__(*args, **kwargs)

        color_scheme_fields = self.init_color_scheme_switch()

        self.helper.layout = Layout(
            TabHolder(
                Tab(_('Main'),
                    'title',
                    'language',
                    'translation_of',
                    'site',
                    css_id='page-main'
                    ),
                Tab(_('Heading'),
                    '_content_title', '_page_title',
                    css_id='page-heading'
                    ),
                Tab(_('Publication'),
                    'active', 'featured', 'publication_date', 'publication_end_date',
                    ),
                Tab(_('Navigation'),
                    'in_navigation', 'parent', 'slug', 'override_url', 'redirect_to',
                    'symlinked_page'
                    ),
                Tab(_('Theme'),
                    'template_key', 'layout', Fieldset(
                        'Themes', 'theme', *color_scheme_fields),
                    css_id='page-theme-settings'
                    ),
            )
        )

        if request:
            _request = copy.copy(request)
            _request.POST = {}

            if kwargs.get('instance', None):
                page = kwargs['instance']

            from .tables import PageDimensionTable
            table = PageDimensionTable(
                _request, page=page, data=page.dimensions, needs_form_wrapper=False)
            dimensions = Tab(_('Dimensions'),
                             HTML(
                table.render()),
                css_id='page-dimensions'

            )
            self.helper.layout[0].append(dimensions)

        self.fields['color_scheme'].required = False


class PageDeleteForm(SelfHandlingForm):

    def handle(self, request, data):
        pass




from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.media.fields import VectorField
from leonardo.module.media.models import Vector
from leonardo.module.web.models import Widget
from leonardo.module.web.widgets.forms import WidgetUpdateForm


class VectorForm(WidgetUpdateForm):

    source = VectorField()


class VectorGraphicsWidget(Widget):

    feincms_item_editor_form = VectorForm

    icon = "fa fa-area-chart"

    source = models.ForeignKey(Vector, related_name="%(app_label)s_%(class)s_related")

    class Meta:
        abstract = True
        verbose_name = _('Vector graphics')
        verbose_name_plural = _('Vector graphics')




from __future__ import absolute_import, unicode_literals

import os
from django.test import TestCase

from django.core import management
from django.utils import six


class ManagementCommandsTest(TestCase):

    def test_01_sync_all(self):

        try:
            # fix sqlite
            from django.db import connection
            connection.cursor()
            connection.connection.text_factory = \
                lambda x: six.text_type(x, "utf-8", "ignore")
        except Exception:
            pass

        management.call_command('sync_all', force=True)

    def test_02_rebuild_index(self):

        management.call_command('rebuild_index', interactive=False)

    def test_03_import_files(self):
        path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)),)
        management.call_command('import_files', path=path)

    """
    in this time is not testable because sqlite hasn't themes
    def test_bootstrap_site(self):
        management.call_command('bootstrap_site',
                                    url='http://github.com/django-leonardo/django-leonardo/raw/develop/contrib/bootstrap/demo.yaml',
                                    sync= True)
        management.call_command('bootstrap_site',
                                options={
                                    'url': 'http://github.com/django-leonardo/django-leonardo/raw/develop/contrib/bootstrap/blog.yaml',
                                    'sync': False,
                                    })
    """




from __future__ import unicode_literals

import os

from django import forms
from django.conf import settings
from django.contrib.admin import widgets
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from .management.commands.import_files import FileImporter
from .models import Clipboard, Folder, FolderRoot, Image, tools


class NewFolderForm(forms.ModelForm):

    class Meta:
        model = Folder
        fields = ('name',)
        widgets = {
            'name': widgets.AdminTextInputWidget,
        }


class ScanFolderForm(forms.Form):

    folders = forms.CharField(
        label='Path to scan', help_text='relative path from MEDIA_ROOT')

    class Meta:
        exclude = ()


def popup_status(request):
    return '_popup' in request.REQUEST or 'pop' in request.REQUEST


def selectfolder_status(request):
    return 'select_folder' in request.REQUEST


def popup_param(request, separator="?"):
    if popup_status(request):
        return "%s_popup=1" % separator
    else:
        return ""


def selectfolder_param(request, separator="&"):
    if selectfolder_status(request):
        return "%sselect_folder=1" % separator
    else:
        return ""


def _userperms(item, request):
    r = []
    ps = ['read', 'edit', 'add_children']
    for p in ps:
        attr = "has_%s_permission" % p
        if hasattr(item, attr):
            x = getattr(item, attr)(request)
            if x:
                r.append(p)
    return r


@login_required
def edit_folder(request, folder_id):
    # TODO: implement edit_folder view
    folder = None
    return render_to_response('admin/media/folder/folder_edit.html', {
        'folder': folder,
        'is_popup': popup_status(request),
        'select_folder': selectfolder_status(request),
    }, context_instance=RequestContext(request))


@login_required
def edit_image(request, folder_id):
    # TODO: implement edit_image view
    folder = None
    return render_to_response('media/image_edit.html', {
        'folder': folder,
        'is_popup': popup_status(request),
        'select_folder': selectfolder_status(request),
    }, context_instance=RequestContext(request))


@login_required
def scan_folder(request, folder_id=None):

    if folder_id:
        folder = Folder.objects.get(id=folder_id)
    else:
        folder = None

    if request.user.is_superuser:
        pass
    elif folder is None:
        # regular users may not add root folders unless configured otherwise
        if not settings.MEDIA_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS:
            raise PermissionDenied
    elif not folder.has_add_children_permission(request):
        # the user does not have the permission to add subfolders
        raise PermissionDenied

    if request.method == 'POST':
        scan_folder_form = ScanFolderForm(request.POST)
        if scan_folder_form.is_valid():
            kwargs = {
                'path': os.path.join(settings.MEDIA_ROOT,
                                     scan_folder_form.cleaned_data['folders']),
            }
            if folder:
                kwargs['base_folder'] = folder.name
            try:
                importer = FileImporter(**kwargs)
                importer.walker(**kwargs)
            except Exception as e:
                raise e
            return render_to_response('admin/media/dismiss_popup.html',
                                      context_instance=RequestContext(request))
    else:
        new_folder_form = ScanFolderForm()
    return render_to_response('admin/media/folder/scan_form.html', {
        'new_folder_form': new_folder_form,
        'is_popup': popup_status(request),
        'select_folder': selectfolder_status(request),
    }, context_instance=RequestContext(request))


@login_required
def make_folder(request, folder_id=None):
    if not folder_id:
        folder_id = request.REQUEST.get('parent_id', None)
    if folder_id:
        folder = Folder.objects.get(id=folder_id)
    else:
        folder = None

    if request.user.is_superuser:
        pass
    elif folder is None:
        # regular users may not add root folders unless configured otherwise
        if not settings.MEDIA_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS:
            raise PermissionDenied
    elif not folder.has_add_children_permission(request):
        # the user does not have the permission to add subfolders
        raise PermissionDenied

    if request.method == 'POST':
        new_folder_form = NewFolderForm(request.POST)
        if new_folder_form.is_valid():
            new_folder = new_folder_form.save(commit=False)
            if (folder or FolderRoot()).contains_folder(new_folder.name):
                new_folder_form._errors['name'] = new_folder_form.error_class(
                    [_('Folder with this name already exists.')])
            else:
                new_folder.parent = folder
                new_folder.owner = request.user
                new_folder.save()
                return render_to_response('admin/media/dismiss_popup.html',
                                          context_instance=RequestContext(request))
    else:
        new_folder_form = NewFolderForm()
    return render_to_response('admin/media/folder/new_folder_form.html', {
        'new_folder_form': new_folder_form,
        'is_popup': popup_status(request),
        'select_folder': selectfolder_status(request),
    }, context_instance=RequestContext(request))


class UploadFileForm(forms.ModelForm):

    class Meta:
        model = Image
        exclude = ()


@login_required
def upload(request):
    return render_to_response('media/upload.html', {
        'title': 'Upload files',
        'is_popup': popup_status(request),
        'select_folder': selectfolder_status(request),
    }, context_instance=RequestContext(request))


@login_required
def paste_clipboard_to_folder(request):
    if request.method == 'POST':
        folder = Folder.objects.get(id=request.POST.get('folder_id'))
        clipboard = Clipboard.objects.get(id=request.POST.get('clipboard_id'))
        if folder.has_add_children_permission(request):
            tools.move_files_from_clipboard_to_folder(clipboard, folder)
            tools.discard_clipboard(clipboard)
        else:
            raise PermissionDenied
    return HttpResponseRedirect('%s?order_by=-modified_at%s%s' % (
                                request.REQUEST.get('redirect_to', ''),
                                popup_param(request, separator='&'),
                                selectfolder_param(request)))


@login_required
def discard_clipboard(request):
    if request.method == 'POST':
        clipboard = Clipboard.objects.get(id=request.POST.get('clipboard_id'))
        tools.discard_clipboard(clipboard)
    return HttpResponseRedirect('%s%s%s' % (
                                request.POST.get('redirect_to', ''),
                                popup_param(request),
                                selectfolder_param(request)))


@login_required
def delete_clipboard(request):
    if request.method == 'POST':
        clipboard = Clipboard.objects.get(id=request.POST.get('clipboard_id'))
        tools.delete_clipboard(clipboard)
    return HttpResponseRedirect('%s%s%s' % (
                                request.POST.get('redirect_to', ''),
                                popup_param(request),
                                selectfolder_param(request)))


@login_required
def clone_files_from_clipboard_to_folder(request):
    if request.method == 'POST':
        clipboard = Clipboard.objects.get(id=request.POST.get('clipboard_id'))
        folder = Folder.objects.get(id=request.POST.get('folder_id'))
        tools.clone_files_from_clipboard_to_folder(clipboard, folder)
    return HttpResponseRedirect('%s%s%s' % (
                                request.POST.get('redirect_to', ''),
                                popup_param(request),
                                selectfolder_param(request)))


def directory_list(request, directory_slug, category_parent_slug):
    if directory_slug is None:
        category_list = Folder.objects.filter(level=0)
        category = None
    else:
        category = Folder.objects.get(name=directory_slug)
        category_list = Folder.objects.filter(parent=category)
    return render_to_response(
        'media/directory_list.html', {
            'object_list': category_list,
            'object': category,
        },
        context_instance=RequestContext(request)
    )


def directory_list_nested(request,
                          directory_slug=None,
                          parent_directory_slug=None,
                          grandparent_directory_slug=None):
    if directory_slug is None:
        object = None
        object_list = Folder.objects.filter(parent=None)
    else:
        if parent_directory_slug is None:
            object = Folder.objects.get(id=directory_slug)
            object_list = object.children.all()
        else:
            if grandparent_directory_slug is None:
                object = Folder.objects.get(
                    name=directory_slug, parent__name=parent_directory_slug)
                object_list = object.children.all()
            else:
                object = Folder.objects.get(
                    name=directory_slug,
                    parent__name=parent_directory_slug,
                    parent__parent__name=grandparent_directory_slug)
                object_list = object.children.all()

    return render_to_response(
        'media/directory_list_nested.html', {
            'object_list': object_list,
            'object': object,
        },
        context_instance=RequestContext(request, {'standalone': True})
    )


def directory_detail_standalone(request, category_id):
    object = Folder.objects.get(id=category_id)

    return render_to_response(
        'media/directory_detail.html', {
            'object': object,
        },
        context_instance=RequestContext(request)
    )
